@@ -57,6 +57,7 @@
 #include "config.h"
 #include "intel_reg.h"
 #include "ioctl_wrappers.h"
+#include "igt_dummyload.h"
 
 /**
  * SECTION:drmtest
@@ -159,12 +160,14 @@ void gem_quiescent_gpu(int fd)
 	struct drm_i915_gem_exec_object2 obj;
 	unsigned ring;
 
+	igt_terminate_spin_batches();
+
 	memset(&obj, 0, sizeof(obj));
 	obj.handle = gem_create(fd, 4096);
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(&bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 
 	for (ring = 0; ring < 1<<6; ring++) {
@@ -180,7 +183,7 @@ void gem_quiescent_gpu(int fd)
 	gem_sync(fd, obj.handle);
 	gem_close(fd, obj.handle);
 
-	igt_drop_caches_set(DROP_RETIRE);
+	igt_drop_caches_set(DROP_RETIRE | DROP_FREED);
 }
 
 /**
@@ -269,6 +272,10 @@ int __drm_open_driver(int chipset)
 		    is_virtio_device(fd))
 			return fd;
 
+		/* Only VGEM-specific tests should be run on VGEM */
+		if (chipset == DRIVER_ANY && !is_vgem_device(fd))
+			return fd;
+
 		close(fd);
 	}
 
@@ -344,14 +351,18 @@ int drm_open_driver(int chipset)
 	fd = __drm_open_driver(chipset);
 	igt_skip_on_f(fd<0, "No known gpu found\n");
 
-	if (__sync_fetch_and_add(&open_count, 1))
-		return fd;
-
+	/* For i915, at least, we ensure that the driver is idle before
+	 * starting a test and we install an exit handler to wait until
+	 * idle before quitting.
+	 */
 	if (is_i915_device(fd)) {
-		gem_quiescent_gpu(fd);
-		igt_install_exit_handler(quiescent_gpu_at_exit);
+		if (__sync_fetch_and_add(&open_count, 1) == 0) {
+			gem_quiescent_gpu(fd);
+
+			at_exit_drm_fd = __drm_open_driver(chipset);
+			igt_install_exit_handler(quiescent_gpu_at_exit);
+		}
 	}
-	at_exit_drm_fd = __drm_open_driver(chipset);
 
 	return fd;
 }
