@@ -62,6 +62,7 @@ bool igt_debugfs_search(const char *filename, const char *substring);
  */
 typedef struct _igt_pipe_crc igt_pipe_crc_t;
 
+#define DRM_MAX_CRC_NR 10
 /**
  * igt_crc_t:
  * @frame: frame number of the capture CRC
@@ -73,8 +74,9 @@ typedef struct _igt_pipe_crc igt_pipe_crc_t;
  */
 typedef struct {
 	uint32_t frame;
+	bool has_valid_frame;
 	int n_words;
-	uint32_t crc[5];
+	uint32_t crc[DRM_MAX_CRC_NR];
 } igt_crc_t;
 
 /**
@@ -111,7 +113,7 @@ enum intel_pipe_crc_source {
         INTEL_PIPE_CRC_SOURCE_MAX,
 };
 
-void igt_assert_crc_equal(igt_crc_t *a, igt_crc_t *b);
+void igt_assert_crc_equal(const igt_crc_t *a, const igt_crc_t *b);
 char *igt_crc_to_string(igt_crc_t *crc);
 
 void igt_require_pipe_crc(void);
@@ -127,6 +129,11 @@ int igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
 			  igt_crc_t **out_crcs);
 void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc);
 
+void igt_hpd_storm_set_threshold(unsigned int threshold);
+void igt_hpd_storm_reset(void);
+bool igt_hpd_storm_detected(void);
+void igt_require_hpd_storm_ctl(void);
+
 /*
  * Drop caches
  */
@@ -162,6 +169,13 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc);
  * Also drop freed objects.
  */
 #define DROP_FREED 0x10
+/**
+ * DROP_SHRINK_ALL:
+ *
+ * Force all unpinned buffers to be evicted from their GTT and returned to the
+ * system.
+ */
+#define DROP_SHRINK_ALL 0x20
 /**
  * DROP_ALL:
  *
@@ -169,10 +183,12 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc);
  */
 #define DROP_ALL (DROP_UNBOUND | \
 		  DROP_BOUND | \
+		  DROP_SHRINK_ALL | \
 		  DROP_RETIRE | \
 		  DROP_ACTIVE | \
 		  DROP_FREED)
 
+bool igt_drop_caches_has(uint64_t val);
 void igt_drop_caches_set(uint64_t val);
 
 /*
