@@ -51,6 +51,7 @@
 #include "intel_io.h"
 #include "drmtest.h"
 #include "igt_aux.h"
+#include "igt_debugfs.h"
 
 /**
  * intel_get_total_ram_mb:
@@ -295,15 +296,29 @@ void intel_purge_vm_caches(void)
 {
 	int fd;
 
+	igt_drop_caches_set(DROP_SHRINK_ALL);
+
+	fd = open("/proc/sys/vm/drop_caches", O_WRONLY);
+	if (fd >= 0) {
+		/* BIT(2): Be quiet. Cannot be combined with other operations,
+		 * the sysctl has a max value of 4.
+		 */
+		igt_ignore_warn(write(fd, "4\n", 2));
+		close(fd);
+	}
+
+	/* Reset write position back to start. Do as a seperate write to keep
+	 * the stages segregated and avoid failure from the squelching stopping
+	 * the purge.
+	 */
 	fd = open("/proc/sys/vm/drop_caches", O_WRONLY);
 	if (fd < 0)
 		return;
 
 	/* BIT(0): Drop page cache
 	 * BIT(1): Drop slab cache
-	 * BIT(2): Be quiet in future
 	 */
-	igt_ignore_warn(write(fd, "7\n", 2));
+	igt_ignore_warn(write(fd, "3\n", 2));
 	close(fd);
 }
 
