@@ -36,6 +36,7 @@
  */
 
 #include "igt.h"
+#include "igt_vgem.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -55,6 +56,7 @@ IGT_TEST_DESCRIPTION("Test of pread/pwrite/mmap behavior when writing to active"
 		     " buffers.");
 
 int fd, devid, gen;
+int vgem_drv = -1;
 int all;
 int pass;
 
@@ -79,7 +81,7 @@ struct buffers {
 	drm_intel_bo **src, **dst;
 	drm_intel_bo *snoop, *spare;
 	uint32_t *tmp;
-	int width, height, npixels;
+	int width, height, npixels, page_size;
 	int count, num_buffers;
 };
 
@@ -241,7 +243,7 @@ static void create_userptr_require(const struct create *create, unsigned count)
 			has_userptr = drmIoctl(fd,
 					 LOCAL_IOCTL_I915_GEM_USERPTR,
 					 &arg) == 0;
-			free((void *)(uintptr_t)arg.user_ptr);
+			free(from_user_pointer(arg.user_ptr));
 		}
 
 	}
@@ -256,13 +258,12 @@ userptr_create_bo(const struct buffers *b)
 	void *ptr;
 
 	memset(&userptr, 0, sizeof(userptr));
-	userptr.user_size = b->npixels * 4;
-	userptr.user_size = (userptr.user_size + 4095) & -4096;
+	userptr.user_size = b->page_size;
 
 	ptr = mmap(NULL, userptr.user_size,
 		   PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
 	igt_assert(ptr != (void *)-1);
-	userptr.user_ptr = (uintptr_t)ptr;
+	userptr.user_ptr = to_user_pointer(ptr);
 
 #if 0
 	do_or_die(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr));
@@ -274,7 +275,7 @@ userptr_create_bo(const struct buffers *b)
 					userptr.user_size, 0);
 	igt_assert(bo);
 #endif
-	bo->virtual = (void *)(uintptr_t)userptr.user_ptr;
+	bo->virtual = from_user_pointer(userptr.user_ptr);
 
 	return bo;
 }
@@ -355,8 +356,7 @@ dmabuf_create_bo(const struct buffers *b)
 	struct dmabuf *dmabuf;
 	int size;
 
-	size = 4*b->npixels;
-	size = (size + 4095) & -4096;
+	size = b->page_size;
 
 	memset(&args, 0, sizeof(args));
 	args.handle = gem_create(fd, size);
@@ -387,12 +387,12 @@ static void
 dmabuf_set_bo(struct buffers *b, drm_intel_bo *bo, uint32_t val)
 {
 	struct dmabuf *dmabuf = bo->virtual;
-	uint32_t *v;
-	int size;
+	uint32_t *v = dmabuf->map;
+	int y;
 
 	prime_sync_start(dmabuf->fd, true);
-	for (v = dmabuf->map, size = b->npixels; size--; v++)
-		*v = val;
+	for (y = 0; y < b->height; y++)
+		v[pixel(y, b->width)] = val;
 	prime_sync_end(dmabuf->fd, true);
 }
 
@@ -400,12 +400,12 @@ static void
 dmabuf_cmp_bo(struct buffers *b, drm_intel_bo *bo, uint32_t val)
 {
 	struct dmabuf *dmabuf = bo->virtual;
-	uint32_t *v;
-	int size;
+	uint32_t *v = dmabuf->map;
+	int y;
 
 	prime_sync_start(dmabuf->fd, false);
-	for (v = dmabuf->map, size = b->npixels; size--; v++)
-		igt_assert_eq_u32(*v, val);
+	for (y = 0; y < b->height; y++)
+		igt_assert_eq_u32(v[pixel(y, b->width)], val);
 	prime_sync_end(dmabuf->fd, false);
 }
 
@@ -423,15 +423,72 @@ dmabuf_release_bo(drm_intel_bo *bo)
 	drm_intel_bo_unreference(bo);
 }
 
+static bool has_prime_export(int _fd)
+{
+	uint64_t value;
+
+	if (drmGetCap(_fd, DRM_CAP_PRIME, &value))
+		return false;
+
+	return value & DRM_PRIME_CAP_EXPORT;
+}
+
+static void create_vgem_require(const struct create *create, unsigned count)
+{
+	igt_require(vgem_drv != -1);
+	igt_require(has_prime_export(vgem_drv));
+	create_dmabuf_require(create, count);
+}
+
+static drm_intel_bo *
+vgem_create_bo(const struct buffers *b)
+{
+	struct drm_prime_handle args;
+	drm_intel_bo *bo;
+	struct vgem_bo vgem;
+	struct dmabuf *dmabuf;
+
+	igt_assert(vgem_drv != -1);
+
+	vgem.width = b->width;
+	vgem.height = b->height;
+	vgem.bpp = 32;
+	vgem_create(vgem_drv, &vgem);
+
+	memset(&args, 0, sizeof(args));
+	args.handle = vgem.handle;
+	args.flags = DRM_RDWR;
+	args.fd = -1;
+
+	do_ioctl(vgem_drv, DRM_IOCTL_PRIME_HANDLE_TO_FD, &args);
+	gem_close(vgem_drv, args.handle);
+	igt_assert(args.fd != -1);
+
+	bo = drm_intel_bo_gem_create_from_prime(b->bufmgr, args.fd, vgem.size);
+	igt_assert(bo);
+
+	dmabuf = malloc(sizeof(*dmabuf));
+	igt_assert(dmabuf);
+
+	dmabuf->fd = args.fd;
+	dmabuf->map = mmap(NULL, vgem.size,
+			   PROT_READ | PROT_WRITE, MAP_SHARED,
+			   dmabuf->fd, 0);
+	igt_assert(dmabuf->map != (void *)-1);
+
+	bo->virtual = dmabuf;
+
+	return bo;
+}
+
 static void
 gtt_set_bo(struct buffers *b, drm_intel_bo *bo, uint32_t val)
 {
 	uint32_t *vaddr = bo->virtual;
-	int size = b->npixels;
 
 	drm_intel_gem_bo_start_gtt_access(bo, true);
-	while (size--)
-		*vaddr++ = val;
+	for (int y = 0; y < b->height; y++)
+		vaddr[pixel(y, b->width)] = val;
 }
 
 static void
@@ -614,9 +671,9 @@ gpu_set_bo(struct buffers *buffers, drm_intel_bo *bo, uint32_t val)
 
 	gem_exec[1].handle = gem_create(fd, 4096);
 	gem_exec[1].relocation_count = 1;
-	gem_exec[1].relocs_ptr = (uintptr_t)reloc;
+	gem_exec[1].relocs_ptr = to_user_pointer(reloc);
 
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = 2;
 	execbuf.batch_len = (b - buf) * sizeof(buf[0]);
 	if (gen >= 6)
@@ -695,7 +752,9 @@ static void buffers_init(struct buffers *b,
 	b->width = size->width;
 	b->height = size->height;
 	b->npixels = size->width * size->height;
-	b->tmp = malloc(4*b->npixels);
+	b->page_size = 4*b->npixels;
+	b->page_size = (b->page_size + 4095) & -4096;
+	b->tmp = malloc(b->page_size);
 	igt_assert(b->tmp);
 
 	b->bufmgr = drm_intel_bufmgr_gem_init(_fd, 4096);
@@ -760,16 +819,6 @@ static void buffers_create(struct buffers *b)
 
 static void buffers_reset(struct buffers *b, bool enable_reuse)
 {
-	buffers_destroy(b);
-
-	igt_assert(b->count == 0);
-	igt_assert(b->tmp);
-	igt_assert(b->src);
-	igt_assert(b->dst);
-
-	intel_batchbuffer_free(b->batch);
-	drm_intel_bufmgr_destroy(b->bufmgr);
-
 	b->bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 	igt_assert(b->bufmgr);
 
@@ -796,7 +845,7 @@ static void buffers_fini(struct buffers *b)
 }
 
 typedef void (*do_copy)(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src);
-typedef struct igt_hang_ring (*do_hang)(void);
+typedef igt_hang_t (*do_hang)(void);
 
 static void render_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 {
@@ -832,7 +881,7 @@ static void blt_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 
 static void cpu_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 {
-	const int size = b->npixels * sizeof(uint32_t);
+	const int size = b->page_size;
 	void *d, *s;
 
 	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_CPU, 0);
@@ -848,7 +897,7 @@ static void cpu_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 
 static void gtt_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 {
-	const int size = b->npixels * sizeof(uint32_t);
+	const int size = b->page_size;
 	void *d, *s;
 
 	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_GTT, 0);
@@ -865,7 +914,7 @@ static void gtt_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 
 static void wc_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 {
-	const int size = b->width * sizeof(uint32_t);
+	const int size = b->page_size;
 	void *d, *s;
 
 	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_GTT, 0);
@@ -880,27 +929,27 @@ static void wc_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 	munmap(s, size);
 }
 
-static struct igt_hang_ring no_hang(void)
+static igt_hang_t no_hang(void)
 {
-	return (struct igt_hang_ring){0, 0};
+	return (igt_hang_t){0, 0};
 }
 
-static struct igt_hang_ring bcs_hang(void)
+static igt_hang_t bcs_hang(void)
 {
 	return igt_hang_ring(fd, I915_EXEC_BLT);
 }
 
-static struct igt_hang_ring rcs_hang(void)
+static igt_hang_t rcs_hang(void)
 {
 	return igt_hang_ring(fd, I915_EXEC_RENDER);
 }
 
-static struct igt_hang_ring all_hang(void)
+static igt_hang_t all_hang(void)
 {
 	uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj;
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	unsigned engine;
 
 	memset(&obj, 0, sizeof(obj));
@@ -908,7 +957,7 @@ static struct igt_hang_ring all_hang(void)
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(&bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 
 	for_each_engine(fd, engine) {
@@ -932,7 +981,7 @@ static void do_basic0(struct buffers *buffers,
 
 	buffers->mode->set_bo(buffers, buffers->src[0], 0xdeadbeef);
 	for (int i = 0; i < buffers->count; i++) {
-		struct igt_hang_ring hang = do_hang_func();
+		igt_hang_t hang = do_hang_func();
 
 		do_copy_func(buffers, buffers->dst[i], buffers->src[0]);
 		buffers->mode->cmp_bo(buffers, buffers->dst[i], 0xdeadbeef);
@@ -948,7 +997,7 @@ static void do_basic1(struct buffers *buffers,
 	gem_quiescent_gpu(fd);
 
 	for (int i = 0; i < buffers->count; i++) {
-		struct igt_hang_ring hang = do_hang_func();
+		igt_hang_t hang = do_hang_func();
 
 		buffers->mode->set_bo(buffers, buffers->src[i], i);
 		buffers->mode->set_bo(buffers, buffers->dst[i], ~i);
@@ -965,7 +1014,7 @@ static void do_basicN(struct buffers *buffers,
 		      do_copy do_copy_func,
 		      do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 
 	gem_quiescent_gpu(fd);
 
@@ -991,7 +1040,7 @@ static void do_overwrite_source(struct buffers *buffers,
 				do_copy do_copy_func,
 				do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1015,7 +1064,7 @@ static void do_overwrite_source_read(struct buffers *buffers,
 				     int do_rcs)
 {
 	const int half = buffers->count/2;
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1059,7 +1108,7 @@ static void do_overwrite_source__rev(struct buffers *buffers,
 				     do_copy do_copy_func,
 				     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1081,7 +1130,7 @@ static void do_overwrite_source__one(struct buffers *buffers,
 				     do_copy do_copy_func,
 				     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 
 	gem_quiescent_gpu(fd);
 	buffers->mode->set_bo(buffers, buffers->src[0], 0);
@@ -1099,7 +1148,7 @@ static void do_intermix(struct buffers *buffers,
 			int do_rcs)
 {
 	const int half = buffers->count/2;
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1153,7 +1202,7 @@ static void do_early_read(struct buffers *buffers,
 			  do_copy do_copy_func,
 			  do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1171,7 +1220,7 @@ static void do_read_read_bcs(struct buffers *buffers,
 			     do_copy do_copy_func,
 			     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1192,7 +1241,7 @@ static void do_write_read_bcs(struct buffers *buffers,
 			      do_copy do_copy_func,
 			      do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1212,7 +1261,7 @@ static void do_read_read_rcs(struct buffers *buffers,
 			     do_copy do_copy_func,
 			     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1233,7 +1282,7 @@ static void do_write_read_rcs(struct buffers *buffers,
 			      do_copy do_copy_func,
 			      do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1253,7 +1302,7 @@ static void do_gpu_read_after_write(struct buffers *buffers,
 				    do_copy do_copy_func,
 				    do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1324,7 +1373,9 @@ static void __run_forked(struct buffers *buffers,
 
 {
 	/* purge the libdrm caches before cloing the process */
-	buffers_reset(buffers, true);
+	buffers_destroy(buffers);
+	intel_batchbuffer_free(buffers->batch);
+	drm_intel_bufmgr_destroy(buffers->bufmgr);
 
 	igt_fork(child, num_children) {
 		int num_buffers;
@@ -1349,6 +1400,8 @@ static void __run_forked(struct buffers *buffers,
 	}
 	igt_waitchildren();
 	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
+
+	buffers_reset(buffers, true);
 }
 
 static void run_forked(struct buffers *buffers,
@@ -1432,138 +1485,142 @@ run_mode(const char *prefix,
 			continue;
 
 		if (!*h->suffix)
-			igt_fork_hang_detector(fd);
+			igt_fixture
+				igt_fork_hang_detector(fd);
 
 		for (p = all ? pipelines : pskip; p->prefix; p++) {
-			igt_fixture p->require();
+			igt_subtest_group  {
+				igt_fixture p->require();
 
-			igt_subtest_f("%s-%s-%s-sanitycheck0%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers, do_basic0,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-sanitycheck0%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers, do_basic0,
+							p->copy, h->hang);
+				}
 
-			igt_subtest_f("%s-%s-%s-sanitycheck1%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers, do_basic1,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-sanitycheck1%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers, do_basic1,
+							p->copy, h->hang);
+				}
 
-			igt_subtest_f("%s-%s-%s-sanitycheckN%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers, do_basicN,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-sanitycheckN%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers, do_basicN,
+							p->copy, h->hang);
+				}
 
-			/* try to overwrite the source values */
-			igt_subtest_f("%s-%s-%s-overwrite-source-one%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_overwrite_source__one,
-					      p->copy, h->hang);
-			}
+				/* try to overwrite the source values */
+				igt_subtest_f("%s-%s-%s-overwrite-source-one%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_overwrite_source__one,
+							p->copy, h->hang);
+				}
 
-			igt_subtest_f("%s-%s-%s-overwrite-source%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_overwrite_source,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-overwrite-source%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_overwrite_source,
+							p->copy, h->hang);
+				}
 
-			igt_subtest_f("%s-%s-%s-overwrite-source-read-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_overwrite_source_read_bcs,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-overwrite-source-read-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_overwrite_source_read_bcs,
+							p->copy, h->hang);
+				}
 
-			igt_subtest_f("%s-%s-%s-overwrite-source-read-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				igt_require(rendercopy);
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_overwrite_source_read_rcs,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-overwrite-source-read-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					igt_require(rendercopy);
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_overwrite_source_read_rcs,
+							p->copy, h->hang);
+				}
 
-			igt_subtest_f("%s-%s-%s-overwrite-source-rev%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_overwrite_source__rev,
-					      p->copy, h->hang);
-			}
+				igt_subtest_f("%s-%s-%s-overwrite-source-rev%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_overwrite_source__rev,
+							p->copy, h->hang);
+				}
 
-			/* try to intermix copies with GPU copies*/
-			igt_subtest_f("%s-%s-%s-intermix-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				igt_require(rendercopy);
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_intermix_rcs,
-					      p->copy, h->hang);
-			}
-			igt_subtest_f("%s-%s-%s-intermix-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				igt_require(rendercopy);
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_intermix_bcs,
-					      p->copy, h->hang);
-			}
-			igt_subtest_f("%s-%s-%s-intermix-both%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				igt_require(rendercopy);
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_intermix_both,
-					      p->copy, h->hang);
-			}
+				/* try to intermix copies with GPU copies*/
+				igt_subtest_f("%s-%s-%s-intermix-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					igt_require(rendercopy);
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_intermix_rcs,
+							p->copy, h->hang);
+				}
+				igt_subtest_f("%s-%s-%s-intermix-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					igt_require(rendercopy);
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_intermix_bcs,
+							p->copy, h->hang);
+				}
+				igt_subtest_f("%s-%s-%s-intermix-both%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					igt_require(rendercopy);
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_intermix_both,
+							p->copy, h->hang);
+				}
 
-			/* try to read the results before the copy completes */
-			igt_subtest_f("%s-%s-%s-early-read%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_early_read,
-					      p->copy, h->hang);
-			}
+				/* try to read the results before the copy completes */
+				igt_subtest_f("%s-%s-%s-early-read%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_early_read,
+							p->copy, h->hang);
+				}
 
-			/* concurrent reads */
-			igt_subtest_f("%s-%s-%s-read-read-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_read_read_bcs,
-					      p->copy, h->hang);
-			}
-			igt_subtest_f("%s-%s-%s-read-read-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				igt_require(rendercopy);
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_read_read_rcs,
-					      p->copy, h->hang);
-			}
+				/* concurrent reads */
+				igt_subtest_f("%s-%s-%s-read-read-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_read_read_bcs,
+							p->copy, h->hang);
+				}
+				igt_subtest_f("%s-%s-%s-read-read-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					igt_require(rendercopy);
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_read_read_rcs,
+							p->copy, h->hang);
+				}
 
-			/* split copying between rings */
-			igt_subtest_f("%s-%s-%s-write-read-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_write_read_bcs,
-					      p->copy, h->hang);
-			}
-			igt_subtest_f("%s-%s-%s-write-read-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				igt_require(rendercopy);
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_write_read_rcs,
-					      p->copy, h->hang);
-			}
+				/* split copying between rings */
+				igt_subtest_f("%s-%s-%s-write-read-bcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_write_read_bcs,
+							p->copy, h->hang);
+				}
+				igt_subtest_f("%s-%s-%s-write-read-rcs%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					igt_require(rendercopy);
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_write_read_rcs,
+							p->copy, h->hang);
+				}
 
-			/* and finally try to trick the kernel into loosing the pending write */
-			igt_subtest_f("%s-%s-%s-gpu-read-after-write%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
-				buffers_create(&buffers);
-				run_wrap_func(&buffers,
-					      do_gpu_read_after_write,
-					      p->copy, h->hang);
+				/* and finally try to trick the kernel into loosing the pending write */
+				igt_subtest_f("%s-%s-%s-gpu-read-after-write%s%s", prefix, mode->name, p->prefix, suffix, h->suffix) {
+					buffers_create(&buffers);
+					run_wrap_func(&buffers,
+							do_gpu_read_after_write,
+							p->copy, h->hang);
+				}
 			}
 		}
 
 		if (!*h->suffix)
-			igt_stop_hang_detector();
+			igt_fixture
+				igt_stop_hang_detector();
 	}
 
 	igt_fixture
@@ -1699,6 +1756,14 @@ igt_main
 			.cmp_bo = dmabuf_cmp_bo,
 			.release_bo = dmabuf_release_bo,
 		},
+		{
+			.name = "vgem",
+			.create_bo = vgem_create_bo,
+			.require = create_vgem_require,
+			.set_bo = dmabuf_set_bo,
+			.cmp_bo = dmabuf_cmp_bo,
+			.release_bo = dmabuf_release_bo,
+		},
 		{
 			.name = "gtt",
 			.create_bo = gtt_create_bo,
@@ -1768,10 +1833,13 @@ igt_main
 		allow_unlimited_files();
 
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 		intel_detect_and_clear_missed_interrupts(fd);
 		devid = intel_get_drm_devid(fd);
 		gen = intel_gen(devid);
 		rendercopy = igt_get_render_copyfunc(devid);
+
+		vgem_drv = __drm_open_driver(DRIVER_VGEM);
 	}
 
 	for (const struct create *c = create; c->name; c++) {
@@ -1830,6 +1898,21 @@ igt_main
 				run_modes(name, c, modes, s, count);
 			}
 
+			snprintf(name, sizeof(name), "%s%s-%s",
+				 c->name, s->name, "shrink");
+			igt_subtest_group {
+				igt_fixture {
+					count = num_buffers(gem_mappable_aperture_size(),
+							    s, c, CHECK_RAM);
+
+					igt_fork_shrink_helper();
+				}
+				run_modes(name, c, modes, s, count);
+
+				igt_fixture
+					igt_stop_shrink_helper();
+			}
+
 			/* Use the entire mappable aperture, force swapping */
 			snprintf(name, sizeof(name), "%s%s-%s",
 				 c->name, s->name, "swap");
