@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -38,16 +39,38 @@
 #include <sys/time.h>
 #include "drm.h"
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
-#include "igt_aux.h"
 
 #define LOCAL_I915_EXEC_VEBOX (4<<0)
 #define LOCAL_I915_EXEC_BSD_MASK (3<<13)
 #define LOCAL_I915_EXEC_BSD_RING1 (1<<13)
 #define LOCAL_I915_EXEC_BSD_RING2 (2<<13)
+#define LOCAL_I915_EXEC_RESOURCE_STREAMER (1<<15)
+#define LOCAL_I915_EXEC_FENCE_IN (1 << 16)
+#define LOCAL_I915_EXEC_FENCE_OUT (1 << 17)
+
+static bool has_ring(int fd, unsigned ring_exec_flags)
+{
+	switch (ring_exec_flags & I915_EXEC_RING_MASK) {
+	case 0:
+	case I915_EXEC_RENDER:
+		return true;
+
+	case I915_EXEC_BSD:
+		if (ring_exec_flags & LOCAL_I915_EXEC_BSD_MASK)
+			return gem_has_bsd2(fd);
+		else
+			return gem_has_bsd(fd);
+
+	case I915_EXEC_BLT:
+		return gem_has_blt(fd);
+
+	case I915_EXEC_VEBOX:
+		return gem_has_vebox(fd);
+	}
+
+	igt_assert_f(0, "invalid exec flag 0x%x\n", ring_exec_flags);
+	return false;
+}
 
 struct drm_i915_gem_execbuffer2 execbuf;
 struct drm_i915_gem_exec_object2 gem_exec[1];
@@ -57,8 +80,11 @@ int fd;
 
 igt_main
 {
+	const struct intel_execution_engine *e;
+
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 
 		devid = intel_get_drm_devid(fd);
 
@@ -74,7 +100,7 @@ igt_main
 		gem_exec[0].rsvd1 = 0;
 		gem_exec[0].rsvd2 = 0;
 
-		execbuf.buffers_ptr = (uintptr_t)gem_exec;
+		execbuf.buffers_ptr = to_user_pointer(gem_exec);
 		execbuf.buffer_count = 1;
 		execbuf.batch_start_offset = 0;
 		execbuf.batch_len = 8;
@@ -88,13 +114,12 @@ igt_main
 	}
 
 	igt_subtest("control") {
-		igt_assert(drmIoctl(fd,
-				    DRM_IOCTL_I915_GEM_EXECBUFFER2,
-				    &execbuf) == 0);
-		execbuf.flags = I915_EXEC_RENDER;
-		igt_assert(drmIoctl(fd,
-				    DRM_IOCTL_I915_GEM_EXECBUFFER2,
-				    &execbuf) == 0);
+		for (e = intel_execution_engines; e->name; e++) {
+			if (has_ring(fd, e->exec_id | e->flags)) {
+				execbuf.flags = e->exec_id | e->flags;
+				gem_execbuf(fd, &execbuf);
+			}
+		}
 	}
 
 #define RUN_FAIL(expected_errno) do { \
@@ -217,10 +242,15 @@ igt_main
 				    &execbuf) == 0);
 	}
 
-	/* HANDLE_LUT and NO_RELOC are already exercised by gem_exec_lut_handle */
+	/* HANDLE_LUT and NO_RELOC are already exercised by gem_exec_lut_handle,
+	 * EXEC_FENCE_IN and EXEC_FENCE_OUT correct usage is tested by
+	 * gem_exec_fence, invalid usage of EXEC_FENCE_IN is tested below. */
 
 	igt_subtest("invalid-flag") {
-		execbuf.flags = I915_EXEC_RENDER | (I915_EXEC_HANDLE_LUT << 1);
+		/* NOTE: This test intentionally exercise the next available
+		 * flag. Don't "fix" this testcase without adding the required
+		 * tests for the new flag first. */
+		execbuf.flags = I915_EXEC_RENDER | (LOCAL_I915_EXEC_FENCE_OUT << 1);
 		RUN_FAIL(EINVAL);
 	}
 
@@ -234,6 +264,47 @@ igt_main
 		execbuf.num_cliprects = 0;
 	}
 
+	igt_subtest("rs-invalid-on-bsd-ring") {
+		igt_require(IS_HASWELL(devid) || intel_gen(devid) >= 8);
+		execbuf.flags = I915_EXEC_BSD | LOCAL_I915_EXEC_RESOURCE_STREAMER;
+		RUN_FAIL(EINVAL);
+	}
+
+	igt_subtest("rs-invalid-on-blt-ring") {
+		igt_require(IS_HASWELL(devid) || intel_gen(devid) >= 8);
+		execbuf.flags = I915_EXEC_BLT | LOCAL_I915_EXEC_RESOURCE_STREAMER;
+		RUN_FAIL(EINVAL);
+	}
+
+	igt_subtest("rs-invalid-on-vebox-ring") {
+		igt_require(IS_HASWELL(devid) || intel_gen(devid) >= 8);
+		execbuf.flags = I915_EXEC_VEBOX | LOCAL_I915_EXEC_RESOURCE_STREAMER;
+		RUN_FAIL(EINVAL);
+	}
+
+	igt_subtest("rs-invalid-gen") {
+		igt_require(!IS_HASWELL(devid) && intel_gen(devid) < 8);
+		execbuf.flags = I915_EXEC_RENDER | LOCAL_I915_EXEC_RESOURCE_STREAMER;
+		RUN_FAIL(EINVAL);
+	}
+
+	igt_subtest("invalid-fence-in") {
+		igt_require(gem_has_exec_fence(fd));
+		execbuf.flags = LOCAL_I915_EXEC_FENCE_IN;
+		execbuf.rsvd2 = -1;
+		RUN_FAIL(EINVAL);
+		execbuf.rsvd2 = fd;
+		RUN_FAIL(EINVAL);
+	}
+
+	igt_subtest("rsvd2-dirt") {
+		igt_require(!gem_has_exec_fence(fd));
+		execbuf.flags = 0;
+		execbuf.rsvd2 = 1;
+		RUN_FAIL(EINVAL);
+		execbuf.rsvd2 = 0;
+	}
+
 #define DIRT(name) \
 	igt_subtest(#name "-dirt") { \
 		execbuf.flags = 0; \
@@ -242,7 +313,6 @@ igt_main
 		execbuf.name = 0; \
 	}
 
-	DIRT(rsvd2);
 	DIRT(cliprects_ptr);
 	DIRT(DR1);
 	DIRT(DR4);
