@@ -26,6 +26,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,18 +40,14 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_debugfs.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Test persistent relocations as used by uxa/libva.");
 
 /*
  * Testcase: Persistent relocations as used by uxa/libva
  *
+ * Note: this currently fails on byt/full-ppgtt
+ * https://bugs.freedesktop.org/show_bug.cgi?id=84859
  */
 
 static drm_intel_bufmgr *bufmgr;
@@ -174,14 +171,14 @@ static void faulting_reloc_and_emit(int fd, drm_intel_bo *target_bo,
 	exec[1].handle = special_bo->handle;
 	exec[1].relocation_count = 1;
 	/* A newly mmap gtt bo will fault on first access. */
-	exec[1].relocs_ptr = (uintptr_t)gtt_relocs;
+	exec[1].relocs_ptr = to_user_pointer(gtt_relocs);
 	exec[1].alignment = 0;
 	exec[1].offset = 0;
 	exec[1].flags = 0;
 	exec[1].rsvd1 = 0;
 	exec[1].rsvd2 = 0;
 
-	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffers_ptr = to_user_pointer(exec);
 	execbuf.buffer_count = 2;
 	execbuf.batch_start_offset = 0;
 	execbuf.batch_len = special_batch_len;
@@ -228,10 +225,8 @@ static void do_test(int fd, bool faulting_reloc)
 
 		relocs_bo_handle[i] = gem_create(fd, 4096);
 		gem_write(fd, relocs_bo_handle[i], 0, reloc, sizeof(reloc));
-		gtt_relocs_ptr[i] = gem_mmap(fd, relocs_bo_handle[i], 4096,
-				      PROT_READ | PROT_WRITE);
-		igt_assert(gtt_relocs_ptr[i]);
-
+		gtt_relocs_ptr[i] = gem_mmap__gtt(fd, relocs_bo_handle[i], 4096,
+						  PROT_READ | PROT_WRITE);
 	}
 
 	/* repeat must be smaller than 4096/small_pitch */
@@ -327,7 +322,7 @@ igt_main
 	memset(blob, 'A', sizeof(blob));
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		/* disable reuse, otherwise the test fails */
