@@ -51,11 +51,6 @@
 
 IGT_TEST_DESCRIPTION("Test of streaming writes into active GPU sources");
 
-static bool __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *eb)
-{
-	return drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, eb) == 0;
-}
-
 #define SRC 0
 #define DST 1
 #define BATCH 2
@@ -106,12 +101,12 @@ static void test_streaming(int fd, int mode, int sync)
 
 	gem_write(fd, dst, 0, tmp, sizeof(tmp));
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffers_ptr = to_user_pointer(exec);
 	execbuf.buffer_count = 2;
 	execbuf.flags = LOCAL_I915_EXEC_HANDLE_LUT;
-	if (!__gem_execbuf(fd, &execbuf)) {
+	if (__gem_execbuf(fd, &execbuf)) {
 		execbuf.flags = 0;
-		igt_require(__gem_execbuf(fd, &execbuf));
+		igt_require(__gem_execbuf(fd, &execbuf) == 0);
 	}
 	/* We assume that the active objects are fixed to avoid relocations */
 	__src_offset = src_offset;
@@ -135,7 +130,7 @@ static void test_streaming(int fd, int mode, int sync)
 		reloc[2*i+1].read_domains = I915_GEM_DOMAIN_RENDER;
 		reloc[2*i+1].write_domain = 0;
 	}
-	igt_assert(__gem_execbuf(fd, &execbuf));
+	gem_execbuf(fd, &execbuf);
 	igt_assert_eq_u64(__src_offset, src_offset);
 	igt_assert_eq_u64(__dst_offset, dst_offset);
 
@@ -154,6 +149,8 @@ static void test_streaming(int fd, int mode, int sync)
 		batch[i].offset = 0;
 
 		base = gem_mmap__cpu(fd, batch[i].handle, 0, 4096, PROT_WRITE);
+		gem_set_domain(fd, batch[i].handle,
+				I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
 
 		for (int j = 0; j < 64; j++) {
 			unsigned x = (n * CHUNK_SIZE) % 4096 >> 2;
@@ -208,7 +205,7 @@ static void test_streaming(int fd, int mode, int sync)
 
 			b = offset / CHUNK_SIZE / 64;
 			n = offset / CHUNK_SIZE % 64;
-			exec[BATCH].relocs_ptr = (uintptr_t)(reloc + 2*n);
+			exec[BATCH].relocs_ptr = to_user_pointer((reloc + 2*n));
 			exec[BATCH].handle = batch[b].handle;
 			exec[BATCH].offset = batch[b].offset;
 			execbuf.batch_start_offset = 64*n;
@@ -243,6 +240,7 @@ static void test_batch(int fd, int mode, int reverse)
 	struct drm_i915_gem_relocation_entry reloc[2];
 	uint32_t tmp[] = { MI_BATCH_BUFFER_END };
 	uint64_t __src_offset, __dst_offset;
+	bool need_64b_start_offset = true;
 	uint64_t batch_size;
 	uint32_t *s, *d;
 	uint32_t *base;
@@ -274,7 +272,7 @@ static void test_batch(int fd, int mode, int reverse)
 	reloc[1].write_domain = 0;
 
 	batch_size = ALIGN(OBJECT_SIZE / CHUNK_SIZE * 128, 4096);
-	exec[BATCH].relocs_ptr = (uintptr_t)reloc;
+	exec[BATCH].relocs_ptr = to_user_pointer(reloc);
 	exec[BATCH].relocation_count = 2;
 	exec[BATCH].handle = gem_create(fd, batch_size);
 
@@ -297,12 +295,12 @@ static void test_batch(int fd, int mode, int reverse)
 
 	gem_write(fd, exec[BATCH].handle, 0, tmp, sizeof(tmp));
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffers_ptr = to_user_pointer(exec);
 	execbuf.buffer_count = 3;
 	execbuf.flags = LOCAL_I915_EXEC_HANDLE_LUT;
 	if (gem_has_blt(fd))
 		execbuf.flags |= I915_EXEC_BLT;
-	if (!__gem_execbuf(fd, &execbuf)) {
+	if (__gem_execbuf(fd, &execbuf)) {
 		execbuf.flags &= ~LOCAL_I915_EXEC_HANDLE_LUT;
 		gem_execbuf(fd, &execbuf);
 	}
@@ -327,7 +325,8 @@ static void test_batch(int fd, int mode, int reverse)
 			int k;
 
 			execbuf.batch_start_offset = 128 * offset;
-			execbuf.batch_start_offset += 8 * (pass & 7);
+			if (!need_64b_start_offset)
+				execbuf.batch_start_offset += 8 * (pass & 7);
 			igt_assert(execbuf.batch_start_offset <= batch_size - 64);
 			if (reverse)
 				execbuf.batch_start_offset = batch_size - execbuf.batch_start_offset - 64;
@@ -375,8 +374,10 @@ igt_main
 {
 	int fd, sync;
 
-	igt_fixture
+	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
+	}
 
 	for (sync = 2; sync--; ) {
 		igt_subtest_f("cpu%s", sync ? "-sync":"")
