@@ -61,17 +61,6 @@
 #define PAGE_SIZE 4096
 #endif
 
-#define LOCAL_I915_GEM_USERPTR       0x33
-#define LOCAL_IOCTL_I915_GEM_USERPTR DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_USERPTR, struct local_i915_gem_userptr)
-struct local_i915_gem_userptr {
-	uint64_t user_ptr;
-	uint64_t user_size;
-	uint32_t flags;
-#define LOCAL_I915_USERPTR_READ_ONLY (1<<0)
-#define LOCAL_I915_USERPTR_UNSYNCHRONIZED (1<<31)
-	uint32_t handle;
-};
-
 static uint32_t userptr_flags = LOCAL_I915_USERPTR_UNSYNCHRONIZED;
 
 #define WIDTH 512
@@ -89,32 +78,6 @@ static void gem_userptr_test_synchronized(void)
 	userptr_flags = 0;
 }
 
-static int gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t *handle)
-{
-	struct local_i915_gem_userptr userptr;
-	int ret;
-
-	memset(&userptr, 0, sizeof(userptr));
-	userptr.user_ptr = (uintptr_t)ptr;
-	userptr.user_size = size;
-	userptr.flags = userptr_flags;
-	if (read_only)
-		userptr.flags |= LOCAL_I915_USERPTR_READ_ONLY;
-
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
-	if (ret)
-		ret = errno;
-	igt_skip_on_f(ret == ENODEV &&
-		      (userptr_flags & LOCAL_I915_USERPTR_UNSYNCHRONIZED) == 0 &&
-		      !read_only,
-		      "Skipping, synchronized mappings with no kernel CONFIG_MMU_NOTIFIER?");
-	if (ret == 0)
-		*handle = userptr.handle;
-
-	return ret;
-}
-
-
 static void gem_userptr_sync(int fd, uint32_t handle)
 {
 	gem_set_domain(fd, handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
@@ -173,36 +136,25 @@ copy(int fd, uint32_t dst, uint32_t src, unsigned int error)
 	reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[1].write_domain = 0;
 
+	memset(&exec, 0, sizeof(exec));
 	memset(obj, 0, sizeof(obj));
-	exec.buffer_count = 0;
 	obj[exec.buffer_count++].handle = dst;
 	if (src != dst)
 		obj[exec.buffer_count++].handle = src;
 	obj[exec.buffer_count].handle = handle;
 	obj[exec.buffer_count].relocation_count = 2;
-	obj[exec.buffer_count].relocs_ptr = (uintptr_t)reloc;
+	obj[exec.buffer_count].relocs_ptr = to_user_pointer(reloc);
 	exec.buffer_count++;
-	exec.buffers_ptr = (uintptr_t)obj;
-
-	exec.batch_start_offset = 0;
-	exec.batch_len = i * 4;
-	exec.DR1 = exec.DR4 = 0;
-	exec.num_cliprects = 0;
-	exec.cliprects_ptr = 0;
+	exec.buffers_ptr = to_user_pointer(obj);
 	exec.flags = HAS_BLT_RING(intel_get_drm_devid(fd)) ? I915_EXEC_BLT : 0;
-	i915_execbuffer2_set_context_id(exec, 0);
-	exec.rsvd2 = 0;
 
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
-	if (ret)
-		ret = errno;
+	ret = __gem_execbuf(fd, &exec);
+	gem_close(fd, handle);
 
 	if (error == ~0)
 		igt_assert_neq(ret, 0);
 	else
-		igt_assert(ret == error);
-
-	gem_close(fd, handle);
+		igt_assert_eq(ret, -error);
 }
 
 static int
@@ -257,28 +209,19 @@ blit(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 	reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[1].write_domain = 0;
 
+	memset(&exec, 0, sizeof(exec));
 	obj = calloc(n_bo + 1, sizeof(*obj));
 	for (n = 0; n < n_bo; n++)
 		obj[n].handle = all_bo[n];
 	obj[n].handle = handle;
 	obj[n].relocation_count = 2;
-	obj[n].relocs_ptr = (uintptr_t)reloc;
+	obj[n].relocs_ptr = to_user_pointer(reloc);
 
-	exec.buffers_ptr = (uintptr_t)obj;
+	exec.buffers_ptr = to_user_pointer(obj);
 	exec.buffer_count = n_bo + 1;
-	exec.batch_start_offset = 0;
-	exec.batch_len = i * 4;
-	exec.DR1 = exec.DR4 = 0;
-	exec.num_cliprects = 0;
-	exec.cliprects_ptr = 0;
 	exec.flags = HAS_BLT_RING(intel_get_drm_devid(fd)) ? I915_EXEC_BLT : 0;
-	i915_execbuffer2_set_context_id(exec, 0);
-	exec.rsvd2 = 0;
-
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
-	if (ret)
-		ret = errno;
 
+	ret = __gem_execbuf(fd, &exec);
 	gem_close(fd, handle);
 	free(obj);
 
@@ -289,10 +232,9 @@ static uint32_t
 create_userptr(int fd, uint32_t val, uint32_t *ptr)
 {
 	uint32_t handle;
-	int i, ret;
+	int i;
 
-	ret = gem_userptr(fd, ptr, sizeof(linear), 0, &handle);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, ptr, sizeof(linear), 0, userptr_flags, &handle);
 	igt_assert(handle != 0);
 
 	/* Fill the BO with dwords starting at val */
@@ -363,7 +305,6 @@ static uint32_t create_userptr_bo(int fd, uint64_t size)
 {
 	void *ptr;
 	uint32_t handle;
-	int ret;
 
 	ptr = mmap(NULL, size,
 		   PROT_READ | PROT_WRITE,
@@ -371,8 +312,7 @@ static uint32_t create_userptr_bo(int fd, uint64_t size)
 		   -1, 0);
 	igt_assert(ptr != MAP_FAILED);
 
-	ret = gem_userptr(fd, (uint32_t *)ptr, size, 0, &handle);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, (uint32_t *)ptr, size, 0, userptr_flags, &handle);
 	add_handle_ptr(handle, ptr, size);
 
 	return handle;
@@ -388,7 +328,7 @@ static void flink_userptr_bo(uint32_t old_handle, uint32_t new_handle)
 		       handle_size_map[old_handle]);
 }
 
-static void clear(int fd, uint32_t handle, int size)
+static void clear(int fd, uint32_t handle, uint64_t size)
 {
 	void *ptr = get_handle_ptr(handle);
 
@@ -450,7 +390,7 @@ static int has_userptr(int fd)
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 	oldflags = userptr_flags;
 	gem_userptr_test_unsynchronized();
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 	userptr_flags = oldflags;
 	if (ret != 0) {
 		free(ptr);
@@ -497,6 +437,10 @@ static int test_input_checking(int fd)
 
 static int test_access_control(int fd)
 {
+	/* CAP_SYS_ADMIN is needed for UNSYNCHRONIZED mappings. */
+	gem_userptr_test_unsynchronized();
+	igt_require(has_userptr(fd));
+
 	igt_fork(child, 1) {
 		void *ptr;
 		int ret;
@@ -504,16 +448,13 @@ static int test_access_control(int fd)
 
 		igt_drop_root();
 
-		/* CAP_SYS_ADMIN is needed for UNSYNCHRONIZED mappings. */
-		gem_userptr_test_unsynchronized();
-
 		igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 
-		ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
+		ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 		if (ret == 0)
 			gem_close(fd, handle);
 		free(ptr);
-		igt_assert(ret == EPERM);
+		igt_assert_eq(ret, -EPERM);
 	}
 
 	igt_waitchildren();
@@ -524,11 +465,9 @@ static int test_access_control(int fd)
 static int test_invalid_null_pointer(int fd)
 {
 	uint32_t handle;
-	int ret;
 
 	/* NULL pointer. */
-	ret = gem_userptr(fd, NULL, PAGE_SIZE, 0, &handle);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, NULL, PAGE_SIZE, 0, userptr_flags, &handle);
 
 	copy(fd, handle, handle, ~0); /* QQQ Precise errno? */
 	gem_close(fd, handle);
@@ -538,24 +477,64 @@ static int test_invalid_null_pointer(int fd)
 
 static int test_invalid_gtt_mapping(int fd)
 {
-	uint32_t handle, handle2;
-	void *ptr;
-	int ret;
+	struct drm_i915_gem_mmap_gtt arg;
+	uint32_t handle;
+	char *gtt, *map;
+
+	/* Anonymous mapping to find a hole */
+	map = mmap(NULL, sizeof(linear) + 2 * PAGE_SIZE,
+		   PROT_READ | PROT_WRITE,
+		   MAP_PRIVATE | MAP_ANONYMOUS,
+		   -1, 0);
+	igt_assert(map != MAP_FAILED);
+
+	gem_userptr(fd, map, sizeof(linear) + 2 * PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, 0);
+	gem_close(fd, handle);
+
+	gem_userptr(fd, map, PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, 0);
+	gem_close(fd, handle);
+
+	gem_userptr(fd, map + sizeof(linear) + PAGE_SIZE, PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, 0);
+	gem_close(fd, handle);
 
 	/* GTT mapping */
-	handle = create_bo(fd, 0);
-	ptr = gem_mmap__gtt(fd, handle, sizeof(linear),
-			    PROT_READ | PROT_WRITE);
-	gem_close(fd, handle);
-	igt_assert(((unsigned long)ptr & (PAGE_SIZE - 1)) == 0);
+	memset(&arg, 0, sizeof(arg));
+	arg.handle = create_bo(fd, 0);
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &arg);
+	gtt = mmap(map + PAGE_SIZE, sizeof(linear),
+		   PROT_READ | PROT_WRITE,
+		   MAP_SHARED | MAP_FIXED,
+		   fd, arg.offset);
+	igt_assert(gtt == map + PAGE_SIZE);
+	gem_close(fd, arg.handle);
+	igt_assert(((unsigned long)gtt & (PAGE_SIZE - 1)) == 0);
 	igt_assert((sizeof(linear) & (PAGE_SIZE - 1)) == 0);
 
-	ret = gem_userptr(fd, ptr, sizeof(linear), 0, &handle2);
-	igt_assert_eq(ret, 0);
-	copy(fd, handle2, handle2, ~0); /* QQQ Precise errno? */
-	gem_close(fd, handle2);
+	gem_userptr(fd, gtt, sizeof(linear), 0, userptr_flags, &handle);
+	copy(fd, handle, handle, EFAULT);
+	gem_close(fd, handle);
 
-	munmap(ptr, sizeof(linear));
+	gem_userptr(fd, gtt, PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, EFAULT);
+	gem_close(fd, handle);
+
+	gem_userptr(fd, gtt + sizeof(linear) - PAGE_SIZE, PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, EFAULT);
+	gem_close(fd, handle);
+
+	/* boundaries */
+	gem_userptr(fd, map, 2*PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, EFAULT);
+	gem_close(fd, handle);
+
+	gem_userptr(fd, map + sizeof(linear), 2*PAGE_SIZE, 0, userptr_flags, &handle);
+	copy(fd, handle, handle, EFAULT);
+	gem_close(fd, handle);
+
+	munmap(map, sizeof(linear) + 2*PAGE_SIZE);
 
 	return 0;
 }
@@ -594,8 +573,7 @@ static void test_forked_access(int fd)
 #ifdef MADV_DONTFORK
 	ret |= madvise(ptr1, sizeof(linear), MADV_DONTFORK);
 #endif
-	ret |= gem_userptr(fd, ptr1, sizeof(linear), 0, &handle1);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, ptr1, sizeof(linear), 0, userptr_flags, &handle1);
 	igt_assert(ptr1);
 	igt_assert(handle1);
 
@@ -603,8 +581,7 @@ static void test_forked_access(int fd)
 #ifdef MADV_DONTFORK
 	ret |= madvise(ptr2, sizeof(linear), MADV_DONTFORK);
 #endif
-	ret |= gem_userptr(fd, ptr2, sizeof(linear), 0, &handle2);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, ptr2, sizeof(linear), 0, userptr_flags, &handle2);
 	igt_assert(ptr2);
 	igt_assert(handle2);
 
@@ -637,46 +614,149 @@ static void test_forked_access(int fd)
 	free(ptr2);
 }
 
+#define MAP_FIXED_INVALIDATE_OVERLAP	(1<<0)
+#define MAP_FIXED_INVALIDATE_BUSY	(1<<1)
+#define MAP_FIXED_INVALIDATE_GET_PAGES	(1<<2)
+#define ALL_MAP_FIXED_INVALIDATE (MAP_FIXED_INVALIDATE_OVERLAP | \
+				  MAP_FIXED_INVALIDATE_BUSY | \
+				  MAP_FIXED_INVALIDATE_GET_PAGES)
+
+static int test_map_fixed_invalidate(int fd, uint32_t flags)
+{
+	const size_t ptr_size = sizeof(linear) + 2*PAGE_SIZE;
+	const int num_handles = (flags & MAP_FIXED_INVALIDATE_OVERLAP) ? 2 : 1;
+	uint32_t handle[num_handles];
+	uint32_t *ptr;
+
+	ptr = mmap(NULL, ptr_size,
+		   PROT_READ | PROT_WRITE,
+		   MAP_SHARED | MAP_ANONYMOUS,
+		   -1, 0);
+	igt_assert(ptr != MAP_FAILED);
+
+	for (int i = 0; i < num_handles; i++)
+		handle[i] = create_userptr(fd, 0, ptr + PAGE_SIZE/sizeof(*ptr));
+
+	for (char *fixed = (char *)ptr, *end = fixed + ptr_size;
+	     fixed + 2*PAGE_SIZE <= end;
+	     fixed += PAGE_SIZE) {
+		struct drm_i915_gem_mmap_gtt mmap_gtt;
+		char *map;
+
+		ptr = mmap(ptr, ptr_size, PROT_READ | PROT_WRITE,
+			   MAP_SHARED | MAP_ANONYMOUS | MAP_FIXED,
+			   -1, 0);
+
+		memset(&mmap_gtt, 0, sizeof(mmap_gtt));
+		mmap_gtt.handle = gem_create(fd, 2*PAGE_SIZE);
+		do_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_gtt);
+
+		if (flags & MAP_FIXED_INVALIDATE_BUSY)
+			copy(fd, handle[0], handle[num_handles-1], 0);
+
+		map = mmap(fixed, 2*PAGE_SIZE,
+			   PROT_READ | PROT_WRITE,
+			   MAP_SHARED | MAP_FIXED,
+			   fd, mmap_gtt.offset);
+		igt_assert(map == fixed);
+
+		gem_set_tiling(fd, mmap_gtt.handle, I915_TILING_NONE, 0);
+		*(uint32_t*)map = 0xdead;
+
+		if (flags & MAP_FIXED_INVALIDATE_GET_PAGES)
+			igt_assert_eq(__gem_set_domain(fd, handle[0],
+						       I915_GEM_DOMAIN_GTT,
+						       I915_GEM_DOMAIN_GTT),
+				      -EFAULT);
+
+		gem_set_tiling(fd, mmap_gtt.handle, I915_TILING_Y, 512 * 4);
+		*(uint32_t*)map = 0xbeef;
+
+		gem_close(fd, mmap_gtt.handle);
+	}
+
+	for (int i = 0; i < num_handles; i++)
+		gem_close(fd, handle[i]);
+	munmap(ptr, ptr_size);
+
+	return 0;
+}
+
 static int test_forbidden_ops(int fd)
 {
 	struct drm_i915_gem_pread gem_pread;
 	struct drm_i915_gem_pwrite gem_pwrite;
-	void *ptr;
-	int ret;
 	uint32_t handle;
-	char buf[PAGE_SIZE];
-
-	memset(&gem_pread, 0, sizeof(gem_pread));
-	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
+	void *ptr;
 
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
-
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 
 	/* pread/pwrite are not always forbidden, but when they
 	 * are they should fail with EINVAL.
 	 */
 
+	memset(&gem_pread, 0, sizeof(gem_pread));
 	gem_pread.handle = handle;
 	gem_pread.offset = 0;
 	gem_pread.size = PAGE_SIZE;
-	gem_pread.data_ptr = (uintptr_t)buf;
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread);
-	igt_assert(ret == 0 || errno == EINVAL);
+	gem_pread.data_ptr = to_user_pointer(ptr);
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread))
+		igt_assert_eq(errno, EINVAL);
 
+	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
 	gem_pwrite.handle = handle;
 	gem_pwrite.offset = 0;
 	gem_pwrite.size = PAGE_SIZE;
-	gem_pwrite.data_ptr = (uintptr_t)buf;
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite);
-	igt_assert(ret == 0 || errno == EINVAL);
+	gem_pwrite.data_ptr = to_user_pointer(ptr);
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite))
+		igt_assert_eq(errno, EINVAL);
+
 	gem_close(fd, handle);
 	free(ptr);
 
 	return 0;
 }
 
+static void test_relocations(int fd)
+{
+	struct drm_i915_gem_relocation_entry *reloc;
+	struct drm_i915_gem_exec_object2 obj;
+	struct drm_i915_gem_execbuffer2 exec;
+	unsigned size;
+	void *ptr;
+	int i;
+
+	size = PAGE_SIZE + ALIGN(sizeof(*reloc)*256, PAGE_SIZE);
+
+	memset(&obj, 0, sizeof(obj));
+	igt_assert(posix_memalign(&ptr, PAGE_SIZE, size) == 0);
+	gem_userptr(fd, ptr, size, 0, userptr_flags, &obj.handle);
+	if (!gem_has_llc(fd))
+		gem_set_caching(fd, obj.handle, 0);
+	*(uint32_t *)ptr = MI_BATCH_BUFFER_END;
+
+	reloc = (typeof(reloc))((char *)ptr + PAGE_SIZE);
+	obj.relocs_ptr = to_user_pointer(reloc);
+	obj.relocation_count = 256;
+
+	memset(reloc, 0, 256*sizeof(*reloc));
+	for (i = 0; i < 256; i++) {
+		reloc[i].offset = 2048 - 4*i;
+		reloc[i].target_handle = obj.handle;
+		reloc[i].read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+	}
+
+	memset(&exec, 0, sizeof(exec));
+	exec.buffers_ptr = to_user_pointer(&obj);
+	exec.buffer_count = 1;
+	gem_execbuf(fd, &exec);
+
+	gem_sync(fd, obj.handle);
+	gem_close(fd, obj.handle);
+	free(ptr);
+}
+
 static unsigned char counter;
 
 static void (* volatile orig_sigbus)(int sig, siginfo_t *info, void *param);
@@ -839,19 +919,19 @@ static int test_usage_restrictions(int fd)
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE * 2) == 0);
 
 	/* Address not aligned. */
-	ret = gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE, 0, &handle);
+	ret = __gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE, 0, userptr_flags, &handle);
 	igt_assert_neq(ret, 0);
 
 	/* Size not rounded to page size. */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE - 1, 0, &handle);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE - 1, 0, userptr_flags, &handle);
 	igt_assert_neq(ret, 0);
 
 	/* Both wrong. */
-	ret = gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE - 1, 0, &handle);
+	ret = __gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE - 1, 0, userptr_flags, &handle);
 	igt_assert_neq(ret, 0);
 
 	/* Read-only not supported. */
-	ret = gem_userptr(fd, (char *)ptr, PAGE_SIZE, 1, &handle);
+	ret = __gem_userptr(fd, (char *)ptr, PAGE_SIZE, 1, userptr_flags, &handle);
 	igt_assert_neq(ret, 0);
 
 	free(ptr);
@@ -873,7 +953,7 @@ static int test_create_destroy(int fd, int time)
 		for (n = 0; n < 1000; n++) {
 			igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 
-			do_or_die(gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle));
+			gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 
 			gem_close(fd, handle);
 			free(ptr);
@@ -1028,6 +1108,7 @@ static void test_forking_evictions(int fd, int size, int count,
 
 static void test_mlocked_evictions(int fd, int size, int count)
 {
+	count = min(256, count/2);
 	mlocked_evictions(fd, &fault_ops, size, count);
 	reset_handle_ptr();
 }
@@ -1065,41 +1146,40 @@ static void test_overlap(int fd, int expected)
 
 	igt_assert(posix_memalign((void *)&ptr, PAGE_SIZE, PAGE_SIZE * 3) == 0);
 
-	ret = gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, &handle);
-	igt_assert_eq(ret, 0);
+	gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, userptr_flags, &handle);
 
 	/* before, no overlap */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle2);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert_eq(ret, 0);
 
 	/* after, no overlap */
-	ret = gem_userptr(fd, ptr + PAGE_SIZE * 2, PAGE_SIZE, 0, &handle2);
+	ret = __gem_userptr(fd, ptr + PAGE_SIZE * 2, PAGE_SIZE, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert_eq(ret, 0);
 
 	/* exactly overlapping */
-	ret = gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, &handle2);
+	ret = __gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
 
 	/* start overlaps */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE * 2, 0, &handle2);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE * 2, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
 
 	/* end overlaps */
-	ret = gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE * 2, 0, &handle2);
+	ret = __gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE * 2, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
 
 	/* subsumes */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE * 3, 0, &handle2);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE * 3, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
@@ -1124,8 +1204,7 @@ static void test_unmap(int fd, int expected)
 	bo_ptr = (char *)ALIGN((unsigned long)ptr, PAGE_SIZE);
 
 	for (i = 0; i < num_obj; i++, bo_ptr += sizeof(linear)) {
-		ret = gem_userptr(fd, bo_ptr, sizeof(linear), 0, &bo[i]);
-		igt_assert_eq(ret, 0);
+		gem_userptr(fd, bo_ptr, sizeof(linear), 0, userptr_flags, &bo[i]);
 	}
 
 	bo[num_obj] = create_bo(fd, 0);
@@ -1159,8 +1238,7 @@ static void test_unmap_after_close(int fd)
 	bo_ptr = (char *)ALIGN((unsigned long)ptr, PAGE_SIZE);
 
 	for (i = 0; i < num_obj; i++, bo_ptr += sizeof(linear)) {
-		ret = gem_userptr(fd, bo_ptr, sizeof(linear), 0, &bo[i]);
-		igt_assert_eq(ret, 0);
+		gem_userptr(fd, bo_ptr, sizeof(linear), 0, userptr_flags, &bo[i]);
 	}
 
 	bo[num_obj] = create_bo(fd, 0);
@@ -1230,8 +1308,7 @@ static void test_stress_mm(int fd)
 	igt_assert_eq(ret, 0);
 
 	while (loops--) {
-		ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
-		igt_assert_eq(ret, 0);
+		gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 
 		gem_close(fd, handle);
 	}
@@ -1265,8 +1342,7 @@ static void *mm_userptr_close_thread(void *data)
 	while (!t->stop) {
 		pthread_mutex_unlock(&t->mutex);
 		for (int i = 0; i < num_handles; i++)
-			igt_assert_eq(gem_userptr(t->fd, t->ptr, PAGE_SIZE, 0, &handle[i]),
-				      0);
+			gem_userptr(t->fd, t->ptr, PAGE_SIZE, 0, userptr_flags, &handle[i]);
 		for (int i = 0; i < num_handles; i++)
 			gem_close(t->fd, handle[i]);
 		pthread_mutex_lock(&t->mutex);
@@ -1305,7 +1381,7 @@ static void test_invalidate_close_race(int fd, bool overlap)
 	free(t_data.ptr);
 }
 
-unsigned int total_ram;
+uint64_t total_ram;
 uint64_t aperture_size;
 int fd, count;
 
@@ -1317,14 +1393,9 @@ int main(int argc, char **argv)
 	igt_subtest_init(argc, argv);
 
 	igt_fixture {
-		int ret;
-
 		fd = drm_open_driver(DRIVER_INTEL);
 		igt_assert(fd >= 0);
-
-		ret = has_userptr(fd);
-		igt_skip_on_f(ret == 0, "No userptr support - %s (%d)\n",
-			      strerror(errno), ret);
+		igt_require_gem(fd);
 
 		size = sizeof(linear);
 
@@ -1337,7 +1408,7 @@ int main(int argc, char **argv)
 			count = 2 * aperture_size / (1024*1024) / 3;
 
 		total_ram = intel_get_total_ram_mb();
-		igt_info("Total RAM is %u MiB\n", total_ram);
+		igt_info("Total RAM is %'llu MiB\n", (long long)total_ram);
 
 		if (count > total_ram * 3 / 4) {
 			count = intel_get_total_ram_mb() * 3 / 4;
@@ -1345,206 +1416,231 @@ int main(int argc, char **argv)
 		}
 	}
 
-	igt_subtest("input-checking")
-		test_input_checking(fd);
-
-	igt_subtest("usage-restrictions")
-		test_usage_restrictions(fd);
-
-	igt_subtest("invalid-null-pointer")
-		test_invalid_null_pointer(fd);
-
-	igt_subtest("invalid-gtt-mapping")
-		test_invalid_gtt_mapping(fd);
-
-	igt_subtest("forked-access")
-		test_forked_access(fd);
-
-	igt_subtest("forbidden-operations")
-		test_forbidden_ops(fd);
-
-	igt_info("Testing unsynchronized mappings...\n");
-	gem_userptr_test_unsynchronized();
-
-	igt_subtest("create-destroy-unsync")
-		test_create_destroy(fd, 5);
-
-	igt_subtest("unsync-overlap")
-		test_overlap(fd, 0);
-
-	igt_subtest("unsync-unmap")
-		test_unmap(fd, 0);
-
-	igt_subtest("unsync-unmap-cycles")
-		test_unmap_cycles(fd, 0);
-
-	igt_subtest("unsync-unmap-after-close")
-		test_unmap_after_close(fd);
-
-	igt_subtest("coherency-unsync")
-		test_coherency(fd, count);
-
-	igt_subtest("dmabuf-unsync")
-		test_dmabuf();
-
-	for (unsigned flags = 0; flags < ALL_FORKING_EVICTIONS + 1; flags++) {
-		igt_subtest_f("forked-unsync%s%s%s-%s",
-		    flags & FORKING_EVICTIONS_SWAPPING ? "-swapping" : "",
-		    flags & FORKING_EVICTIONS_DUP_DRMFD ? "-multifd" : "",
-		    flags & FORKING_EVICTIONS_MEMORY_PRESSURE ?
-				"-mempressure" : "",
-		    flags & FORKING_EVICTIONS_INTERRUPTIBLE ?
-				"interruptible" : "normal") {
-			test_forking_evictions(fd, size, count, flags);
+	igt_subtest_group {
+		igt_fixture {
+			igt_require(has_userptr(fd));
 		}
+
+		igt_subtest("input-checking")
+			test_input_checking(fd);
+
+		igt_subtest("usage-restrictions")
+			test_usage_restrictions(fd);
+
+		igt_subtest("invalid-null-pointer")
+			test_invalid_null_pointer(fd);
+
+		igt_subtest("invalid-gtt-mapping")
+			test_invalid_gtt_mapping(fd);
+
+		igt_subtest("forked-access")
+			test_forked_access(fd);
+
+		igt_subtest("forbidden-operations")
+			test_forbidden_ops(fd);
+
+		igt_subtest("relocations")
+			test_relocations(fd);
 	}
 
-	igt_subtest("mlocked-unsync-normal")
-		test_mlocked_evictions(fd, size, count);
+	igt_subtest_group {
+		gem_userptr_test_unsynchronized();
 
-	igt_subtest("swapping-unsync-normal")
-		test_swapping_evictions(fd, size, count);
-
-	igt_subtest("minor-unsync-normal")
-		test_minor_evictions(fd, size, count);
-
-	igt_subtest("major-unsync-normal") {
-		size = 200 * 1024 * 1024;
-		count = (gem_aperture_size(fd) / size) + 2;
-		test_major_evictions(fd, size, count);
-	}
-
-	igt_fixture {
-		size = sizeof(linear);
-		count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
-		if (count > total_ram * 3 / 4)
-			count = intel_get_total_ram_mb() * 3 / 4;
-	}
-
-	igt_fork_signal_helper();
-
-	igt_subtest("mlocked-unsync-interruptible")
-		test_mlocked_evictions(fd, size, count);
-
-	igt_subtest("swapping-unsync-interruptible")
-		test_swapping_evictions(fd, size, count);
-
-	igt_subtest("minor-unsync-interruptible")
-		test_minor_evictions(fd, size, count);
-
-	igt_subtest("major-unsync-interruptible") {
-		size = 200 * 1024 * 1024;
-		count = (gem_aperture_size(fd) / size) + 2;
-		test_major_evictions(fd, size, count);
-	}
-
-	igt_stop_signal_helper();
-
-	igt_info("Testing synchronized mappings...\n");
-
-	igt_fixture {
-		size = sizeof(linear);
-		count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
-		if (count > total_ram * 3 / 4)
-			count = intel_get_total_ram_mb() * 3 / 4;
-	}
-
-	gem_userptr_test_synchronized();
-
-	igt_subtest("process-exit")
-		test_process_exit(fd, 0);
-
-	igt_subtest("process-exit-gtt")
-		test_process_exit(fd, PE_GTT_MAP);
-
-	igt_subtest("process-exit-busy")
-		test_process_exit(fd, PE_BUSY);
-
-	igt_subtest("process-exit-gtt-busy")
-		test_process_exit(fd, PE_GTT_MAP | PE_BUSY);
-
-	igt_subtest("create-destroy-sync")
-		test_create_destroy(fd, 5);
-
-	igt_subtest("sync-overlap")
-		test_overlap(fd, EINVAL);
-
-	igt_subtest("sync-unmap")
-		test_unmap(fd, EFAULT);
-
-	igt_subtest("sync-unmap-cycles")
-		test_unmap_cycles(fd, EFAULT);
-
-	igt_subtest("sync-unmap-after-close")
-		test_unmap_after_close(fd);
-
-	igt_subtest("stress-mm")
-		test_stress_mm(fd);
-
-	igt_subtest("stress-mm-invalidate-close")
-		test_invalidate_close_race(fd, false);
-
-	igt_subtest("stress-mm-invalidate-close-overlap")
-		test_invalidate_close_race(fd, true);
-
-	igt_subtest("coherency-sync")
-		test_coherency(fd, count);
-
-	igt_subtest("dmabuf-sync")
-		test_dmabuf();
-
-	for (unsigned flags = 0; flags < ALL_FORKING_EVICTIONS + 1; flags++) {
-		igt_subtest_f("forked-sync%s%s%s-%s",
-		    flags & FORKING_EVICTIONS_SWAPPING ? "-swapping" : "",
-		    flags & FORKING_EVICTIONS_DUP_DRMFD ? "-multifd" : "",
-		    flags & FORKING_EVICTIONS_MEMORY_PRESSURE ?
-				"-mempressure" : "",
-		    flags & FORKING_EVICTIONS_INTERRUPTIBLE ?
-				"interruptible" : "normal") {
-			test_forking_evictions(fd, size, count, flags);
+		igt_fixture {
+			igt_require(has_userptr(fd));
 		}
+
+		igt_subtest("create-destroy-unsync")
+			test_create_destroy(fd, 5);
+
+		igt_subtest("unsync-overlap")
+			test_overlap(fd, 0);
+
+		igt_subtest("unsync-unmap")
+			test_unmap(fd, 0);
+
+		igt_subtest("unsync-unmap-cycles")
+			test_unmap_cycles(fd, 0);
+
+		igt_subtest("unsync-unmap-after-close")
+			test_unmap_after_close(fd);
+
+		igt_subtest("coherency-unsync")
+			test_coherency(fd, count);
+
+		igt_subtest("dmabuf-unsync")
+			test_dmabuf();
+
+		for (unsigned flags = 0; flags < ALL_FORKING_EVICTIONS + 1; flags++) {
+			igt_subtest_f("forked-unsync%s%s%s-%s",
+					flags & FORKING_EVICTIONS_SWAPPING ? "-swapping" : "",
+					flags & FORKING_EVICTIONS_DUP_DRMFD ? "-multifd" : "",
+					flags & FORKING_EVICTIONS_MEMORY_PRESSURE ?
+					"-mempressure" : "",
+					flags & FORKING_EVICTIONS_INTERRUPTIBLE ?
+					"interruptible" : "normal") {
+				test_forking_evictions(fd, size, count, flags);
+			}
+		}
+
+		igt_subtest("mlocked-unsync-normal")
+			test_mlocked_evictions(fd, size, count);
+
+		igt_subtest("swapping-unsync-normal")
+			test_swapping_evictions(fd, size, count);
+
+		igt_subtest("minor-unsync-normal")
+			test_minor_evictions(fd, size, count);
+
+		igt_subtest("major-unsync-normal") {
+			size = 200 * 1024 * 1024;
+			count = (gem_aperture_size(fd) / size) + 2;
+			test_major_evictions(fd, size, count);
+		}
+
+		igt_fixture {
+			size = sizeof(linear);
+			count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
+			if (count > total_ram * 3 / 4)
+				count = intel_get_total_ram_mb() * 3 / 4;
+		}
+
+		igt_fork_signal_helper();
+
+		igt_subtest("mlocked-unsync-interruptible")
+			test_mlocked_evictions(fd, size, count);
+
+		igt_subtest("swapping-unsync-interruptible")
+			test_swapping_evictions(fd, size, count);
+
+		igt_subtest("minor-unsync-interruptible")
+			test_minor_evictions(fd, size, count);
+
+		igt_subtest("major-unsync-interruptible") {
+			size = 200 * 1024 * 1024;
+			count = (gem_aperture_size(fd) / size) + 2;
+			test_major_evictions(fd, size, count);
+		}
+
+		igt_stop_signal_helper();
 	}
 
-	igt_subtest("mlocked-normal-sync")
-		test_mlocked_evictions(fd, size, count);
+	igt_subtest_group {
+		gem_userptr_test_synchronized();
 
-	igt_subtest("swapping-normal-sync")
-		test_swapping_evictions(fd, size, count);
+		igt_fixture {
+			igt_require(has_userptr(fd));
+			size = sizeof(linear);
+			count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
+			if (count > total_ram * 3 / 4)
+				count = intel_get_total_ram_mb() * 3 / 4;
+		}
 
-	igt_subtest("minor-normal-sync")
-		test_minor_evictions(fd, size, count);
+		igt_subtest("process-exit")
+			test_process_exit(fd, 0);
 
-	igt_subtest("major-normal-sync") {
-		size = 200 * 1024 * 1024;
-		count = (gem_aperture_size(fd) / size) + 2;
-		test_major_evictions(fd, size, count);
+		igt_subtest("process-exit-gtt")
+			test_process_exit(fd, PE_GTT_MAP);
+
+		igt_subtest("process-exit-busy")
+			test_process_exit(fd, PE_BUSY);
+
+		igt_subtest("process-exit-gtt-busy")
+			test_process_exit(fd, PE_GTT_MAP | PE_BUSY);
+
+		igt_subtest("create-destroy-sync")
+			test_create_destroy(fd, 5);
+
+		igt_subtest("sync-overlap")
+			test_overlap(fd, EINVAL);
+
+		igt_subtest("sync-unmap")
+			test_unmap(fd, EFAULT);
+
+		igt_subtest("sync-unmap-cycles")
+			test_unmap_cycles(fd, EFAULT);
+
+		igt_subtest("sync-unmap-after-close")
+			test_unmap_after_close(fd);
+
+		igt_subtest("stress-mm")
+			test_stress_mm(fd);
+
+		igt_subtest("stress-mm-invalidate-close")
+			test_invalidate_close_race(fd, false);
+
+		igt_subtest("stress-mm-invalidate-close-overlap")
+			test_invalidate_close_race(fd, true);
+
+		for (unsigned flags = 0; flags < ALL_MAP_FIXED_INVALIDATE + 1; flags++) {
+			igt_subtest_f("map-fixed-invalidate%s%s%s",
+				      flags & MAP_FIXED_INVALIDATE_OVERLAP ? "-overlap" : "",
+				      flags & MAP_FIXED_INVALIDATE_BUSY ? "-busy" : "",
+				      flags & MAP_FIXED_INVALIDATE_GET_PAGES ? "-gup" : "") {
+				test_map_fixed_invalidate(fd, flags);
+			}
+		}
+
+		igt_subtest("coherency-sync")
+			test_coherency(fd, count);
+
+		igt_subtest("dmabuf-sync")
+			test_dmabuf();
+
+		for (unsigned flags = 0; flags < ALL_FORKING_EVICTIONS + 1; flags++) {
+			igt_subtest_f("forked-sync%s%s%s-%s",
+					flags & FORKING_EVICTIONS_SWAPPING ? "-swapping" : "",
+					flags & FORKING_EVICTIONS_DUP_DRMFD ? "-multifd" : "",
+					flags & FORKING_EVICTIONS_MEMORY_PRESSURE ?
+					"-mempressure" : "",
+					flags & FORKING_EVICTIONS_INTERRUPTIBLE ?
+					"interruptible" : "normal") {
+				test_forking_evictions(fd, size, count, flags);
+			}
+		}
+
+		igt_subtest("mlocked-normal-sync")
+			test_mlocked_evictions(fd, size, count);
+
+		igt_subtest("swapping-normal-sync")
+			test_swapping_evictions(fd, size, count);
+
+		igt_subtest("minor-normal-sync")
+			test_minor_evictions(fd, size, count);
+
+		igt_subtest("major-normal-sync") {
+			size = 200 * 1024 * 1024;
+			count = (gem_aperture_size(fd) / size) + 2;
+			test_major_evictions(fd, size, count);
+		}
+
+		igt_fixture {
+			size = 1024 * 1024;
+			count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
+			if (count > total_ram * 3 / 4)
+				count = intel_get_total_ram_mb() * 3 / 4;
+		}
+
+		igt_fork_signal_helper();
+
+		igt_subtest("mlocked-sync-interruptible")
+			test_mlocked_evictions(fd, size, count);
+
+		igt_subtest("swapping-sync-interruptible")
+			test_swapping_evictions(fd, size, count);
+
+		igt_subtest("minor-sync-interruptible")
+			test_minor_evictions(fd, size, count);
+
+		igt_subtest("major-sync-interruptible") {
+			size = 200 * 1024 * 1024;
+			count = (gem_aperture_size(fd) / size) + 2;
+			test_major_evictions(fd, size, count);
+		}
+
+		igt_stop_signal_helper();
 	}
 
-	igt_fixture {
-		size = 1024 * 1024;
-		count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
-		if (count > total_ram * 3 / 4)
-			count = intel_get_total_ram_mb() * 3 / 4;
-	}
-
-	igt_fork_signal_helper();
-
-	igt_subtest("mlocked-sync-interruptible")
-		test_mlocked_evictions(fd, size, count);
-
-	igt_subtest("swapping-sync-interruptible")
-		test_swapping_evictions(fd, size, count);
-
-	igt_subtest("minor-sync-interruptible")
-		test_minor_evictions(fd, size, count);
-
-	igt_subtest("major-sync-interruptible") {
-		size = 200 * 1024 * 1024;
-		count = (gem_aperture_size(fd) / size) + 2;
-		test_major_evictions(fd, size, count);
-	}
-
-	igt_stop_signal_helper();
 
 	igt_subtest("access-control")
 		test_access_control(fd);
