@@ -23,7 +23,9 @@
 
 #include "igt.h"
 #include "drmtest.h"
+#include "sw_sync.h"
 #include <errno.h>
+#include <pthread.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
@@ -43,6 +45,49 @@ struct plane_parms {
 
 #define hweight32 __builtin_popcount
 
+static void
+run_primary_test(igt_display_t *display, enum pipe pipe, igt_output_t *output)
+{
+	drmModeModeInfo *mode;
+	igt_plane_t *primary;
+	igt_fb_t fb;
+	int i, ret;
+	unsigned flags = DRM_MODE_ATOMIC_TEST_ONLY | DRM_MODE_ATOMIC_ALLOW_MODESET;
+
+	igt_output_set_pipe(output, pipe);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+
+	mode = igt_output_get_mode(output);
+
+	igt_plane_set_fb(primary, NULL);
+	ret = igt_display_try_commit_atomic(display, flags, NULL);
+	igt_skip_on_f(ret == -EINVAL, "Primary plane cannot be disabled separately from output\n");
+
+	igt_create_fb(display->drm_fd, mode->hdisplay, mode->vdisplay,
+		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &fb);
+
+	igt_plane_set_fb(primary, &fb);
+
+	for (i = 0; i < 4; i++) {
+		igt_display_commit2(display, COMMIT_ATOMIC);
+
+		if (!(i & 1))
+			igt_wait_for_vblank(display->drm_fd, pipe);
+
+		igt_plane_set_fb(primary, (i & 1) ? &fb : NULL);
+		igt_display_commit2(display, COMMIT_ATOMIC);
+
+		if (i & 1)
+			igt_wait_for_vblank(display->drm_fd, pipe);
+
+		igt_plane_set_fb(primary, (i & 1) ? NULL : &fb);
+	}
+
+	igt_plane_set_fb(primary, NULL);
+	igt_output_set_pipe(output, PIPE_NONE);
+	igt_remove_fb(display->drm_fd, &fb);
+}
+
 static void
 wm_setup_plane(igt_display_t *display, enum pipe pipe,
 	       uint32_t mask, struct plane_parms *parms)
@@ -111,6 +156,236 @@ enum transition_type {
 	TRANSITION_MODESET_DISABLE,
 };
 
+static void set_sprite_wh(igt_display_t *display, enum pipe pipe,
+			  struct plane_parms *parms, struct igt_fb *sprite_fb,
+			  bool alpha, unsigned w, unsigned h)
+{
+	igt_plane_t *plane;
+
+	for_each_plane_on_pipe(display, pipe, plane) {
+		int i = plane->index;
+
+		if (plane->type == DRM_PLANE_TYPE_PRIMARY ||
+		    plane->type == DRM_PLANE_TYPE_CURSOR)
+			continue;
+
+		parms[i].width = w;
+		parms[i].height = h;
+	}
+
+	igt_remove_fb(display->drm_fd, sprite_fb);
+	igt_create_fb(display->drm_fd, w, h,
+		      alpha ? DRM_FORMAT_ARGB8888 : DRM_FORMAT_XRGB8888,
+		      LOCAL_DRM_FORMAT_MOD_NONE, sprite_fb);
+}
+
+static void setup_parms(igt_display_t *display, enum pipe pipe,
+			const drmModeModeInfo *mode,
+			struct igt_fb *argb_fb,
+			struct igt_fb *sprite_fb,
+			struct plane_parms *parms)
+{
+	uint64_t cursor_width, cursor_height;
+	unsigned sprite_width, sprite_height, prev_w, prev_h;
+	bool max_sprite_width, max_sprite_height, alpha = true;
+	uint32_t n_planes = display->pipes[pipe].n_planes;
+	igt_plane_t *plane;
+
+	do_or_die(drmGetCap(display->drm_fd, DRM_CAP_CURSOR_WIDTH, &cursor_width));
+	if (cursor_width >= mode->hdisplay)
+		cursor_width = mode->hdisplay;
+
+	do_or_die(drmGetCap(display->drm_fd, DRM_CAP_CURSOR_HEIGHT, &cursor_height));
+	if (cursor_height >= mode->vdisplay)
+		cursor_height = mode->vdisplay;
+
+	for_each_plane_on_pipe(display, pipe, plane) {
+		int i = plane->index;
+
+		if (plane->type == DRM_PLANE_TYPE_PRIMARY) {
+			parms[i].fb = plane->fb;
+			parms[i].width = mode->hdisplay;
+			parms[i].height = mode->vdisplay;
+		} else if (plane->type == DRM_PLANE_TYPE_CURSOR) {
+			parms[i].fb = argb_fb;
+			parms[i].width = cursor_width;
+			parms[i].height = cursor_height;
+		} else
+			parms[i].fb = sprite_fb;
+	}
+
+	igt_create_fb(display->drm_fd, cursor_width, cursor_height,
+		      DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE, argb_fb);
+
+	igt_create_fb(display->drm_fd, cursor_width, cursor_height,
+		      DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE, sprite_fb);
+
+	if (n_planes < 3)
+		return;
+
+	/*
+	 * Pre gen9 not all sizes are supported, find the biggest possible
+	 * size that can be enabled on all sprite planes.
+	 */
+retry:
+	prev_w = sprite_width = cursor_width;
+	prev_h = sprite_height = cursor_height;
+
+	max_sprite_width = (sprite_width == mode->hdisplay);
+	max_sprite_height = (sprite_height == mode->vdisplay);
+
+	while (1) {
+		int ret;
+
+		set_sprite_wh(display, pipe, parms, sprite_fb,
+			      alpha, sprite_width, sprite_height);
+
+		wm_setup_plane(display, pipe, (1 << n_planes) - 1, parms);
+		ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_TEST_ONLY | DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
+
+		if (ret == -EINVAL) {
+			if (cursor_width == sprite_width &&
+			    cursor_height == sprite_height) {
+				igt_assert_f(alpha,
+					      "Cannot configure the test with all sprite planes enabled\n");
+
+				/* retry once with XRGB format. */
+				alpha = false;
+				goto retry;
+			}
+
+			sprite_width = prev_w;
+			sprite_height = prev_h;
+
+			if (max_sprite_width && max_sprite_height) {
+				set_sprite_wh(display, pipe, parms, sprite_fb,
+					      alpha, sprite_width, sprite_height);
+				break;
+			}
+
+			if (!max_sprite_width)
+				max_sprite_width = true;
+			else
+				max_sprite_height = true;
+		} else {
+			prev_w = sprite_width;
+			prev_h = sprite_height;
+		}
+
+		if (!max_sprite_width) {
+			sprite_width *= 2;
+
+			if (sprite_width >= mode->hdisplay) {
+				max_sprite_width = true;
+
+				sprite_width = mode->hdisplay;
+			}
+		} else if (!max_sprite_height) {
+			sprite_height *= 2;
+
+			if (sprite_height >= mode->vdisplay) {
+				max_sprite_height = true;
+
+				sprite_height = mode->vdisplay;
+			}
+		} else
+			/* Max sized sprites for all! */
+			break;
+	}
+
+	igt_info("Running test on pipe %s with resolution %dx%d and sprite size %dx%d alpha %i\n",
+		 kmstest_pipe_name(pipe), mode->hdisplay, mode->vdisplay,
+		 sprite_width, sprite_height, alpha);
+}
+
+int *timeline;
+pthread_t *thread;
+int *seqno;
+
+static void prepare_fencing(igt_display_t *display, enum pipe pipe)
+{
+	igt_plane_t *plane;
+	int n_planes;
+
+	igt_require_sw_sync();
+
+	n_planes = display->pipes[pipe].n_planes;
+	timeline = calloc(sizeof(*timeline), n_planes);
+	igt_assert_f(timeline != NULL, "Failed to allocate memory for timelines\n");
+	thread = calloc(sizeof(*thread), n_planes);
+	igt_assert_f(thread != NULL, "Failed to allocate memory for thread\n");
+	seqno = calloc(sizeof(*seqno), n_planes);
+	igt_assert_f(seqno != NULL, "Failed to allocate memory for seqno\n");
+
+	for_each_plane_on_pipe(display, pipe, plane)
+		timeline[plane->index] = sw_sync_timeline_create();
+}
+
+static void unprepare_fencing(igt_display_t *display, enum pipe pipe)
+{
+	igt_plane_t *plane;
+
+	for_each_plane_on_pipe(display, pipe, plane)
+		close(timeline[plane->index]);
+
+	free(timeline);
+	free(thread);
+	free(seqno);
+}
+
+static void *fence_inc_thread(void *arg)
+{
+	int t = *((int *) arg);
+
+	pthread_detach(pthread_self());
+
+	usleep(5000);
+	sw_sync_timeline_inc(t, 1);
+	return NULL;
+}
+
+static void configure_fencing(igt_display_t *display, enum pipe pipe)
+{
+	igt_plane_t *plane;
+	int i, fd, ret;
+
+	for_each_plane_on_pipe(display, pipe, plane) {
+
+		if (!plane->fb)
+			continue;
+
+		i = plane->index;
+
+		seqno[i]++;
+		fd = sw_sync_timeline_create_fence(timeline[i], seqno[i]);
+		igt_plane_set_fence_fd(plane, fd);
+		close(fd);
+		ret = pthread_create(&thread[i], NULL, fence_inc_thread, &timeline[i]);
+		igt_assert_eq(ret, 0);
+	}
+}
+
+static void clear_fencing(igt_display_t *display, enum pipe pipe)
+{
+	igt_plane_t *plane;
+
+	for_each_plane_on_pipe(display, pipe, plane)
+		igt_plane_set_fence_fd(plane, -1);
+}
+
+static void atomic_commit(igt_display_t *display, enum pipe pipe, unsigned int flags, void *data, bool fencing)
+{
+	if (fencing) {
+		configure_fencing(display, pipe);
+		igt_pipe_request_out_fence(&display->pipes[pipe]);
+	}
+
+	igt_display_commit_atomic(display, flags, data);
+
+	if (fencing)
+		clear_fencing(display, pipe);
+}
+
 /*
  * 1. Set primary plane to a known fb.
  * 2. Make sure getcrtc returns the correct fb id.
@@ -121,17 +396,20 @@ enum transition_type {
  * so test this and make sure it works.
  */
 static void
-run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output, enum transition_type type, bool nonblocking)
+run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output,
+		enum transition_type type, bool nonblocking, bool fencing)
 {
-	struct igt_fb fb, argb_fb;
+	struct igt_fb fb, argb_fb, sprite_fb;
 	drmModeModeInfo *mode, override_mode;
 	igt_plane_t *plane;
-	uint64_t cursor_width, cursor_height;
-	uint32_t n_planes = display->pipes[pipe].n_planes;
-	uint32_t iter_max = 1 << n_planes, i;
-	struct plane_parms parms[IGT_MAX_PLANES];
+	uint32_t iter_max = 1 << display->pipes[pipe].n_planes, i;
+	struct plane_parms parms[display->pipes[pipe].n_planes];
 	bool skip_test = false;
 	unsigned flags = DRM_MODE_PAGE_FLIP_EVENT;
+	int ret;
+
+	if (fencing)
+		prepare_fencing(display, pipe);
 
 	if (nonblocking)
 		flags |= DRM_MODE_ATOMIC_NONBLOCK;
@@ -146,17 +424,6 @@ run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output
 	igt_create_fb(display->drm_fd, mode->hdisplay, mode->vdisplay,
 		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &fb);
 
-	igt_create_fb(display->drm_fd, mode->hdisplay, mode->vdisplay,
-		      DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &argb_fb);
-
-	do_or_die(drmGetCap(display->drm_fd, DRM_CAP_CURSOR_WIDTH, &cursor_width));
-	if (cursor_width > mode->hdisplay)
-		cursor_width = mode->hdisplay;
-
-	do_or_die(drmGetCap(display->drm_fd, DRM_CAP_CURSOR_HEIGHT, &cursor_height));
-	if (cursor_height > mode->vdisplay)
-		cursor_height = mode->vdisplay;
-
 	igt_output_set_pipe(output, pipe);
 
 	wm_setup_plane(display, pipe, 0, NULL);
@@ -173,44 +440,31 @@ run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output
 		igt_output_set_pipe(output, pipe);
 	}
 
-	for_each_plane_on_pipe(display, pipe, plane) {
-		i = plane->index;
-
-		if (plane->is_primary)
-			parms[i].fb = &fb;
-		else
-			parms[i].fb = &argb_fb;
-
-		if (plane->is_cursor) {
-			parms[i].width = cursor_width;
-			parms[i].height = cursor_height;
-		} else {
-			parms[i].width = mode->hdisplay;
-			parms[i].height = mode->vdisplay;
-		}
-	}
-
 	igt_display_commit2(display, COMMIT_ATOMIC);
 
+	setup_parms(display, pipe, mode, &argb_fb, &sprite_fb, parms);
+
 	/*
 	 * In some configurations the tests may not run to completion with all
 	 * sprite planes lit up at 4k resolution, try decreasing width/size of secondary
 	 * planes to fix this
 	 */
 	while (1) {
-		int ret;
-
 		wm_setup_plane(display, pipe, iter_max - 1, parms);
-		ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_TEST_ONLY | DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
 
-		if (ret != -EINVAL || n_planes < 3)
+		if (fencing)
+			igt_pipe_request_out_fence(&display->pipes[pipe]);
+
+		ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_TEST_ONLY | DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
+		if (ret != -EINVAL || display->pipes[pipe].n_planes < 3)
 			break;
 
 		ret = 0;
 		for_each_plane_on_pipe(display, pipe, plane) {
 			i = plane->index;
 
-			if (plane->is_primary || plane->is_cursor)
+			if (plane->type == DRM_PLANE_TYPE_PRIMARY ||
+			    plane->type == DRM_PLANE_TYPE_CURSOR)
 				continue;
 
 			if (parms[i].width <= 512)
@@ -231,7 +485,7 @@ run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output
 
 		wm_setup_plane(display, pipe, i, parms);
 
-		igt_display_commit_atomic(display, flags, (void *)(unsigned long)i);
+		atomic_commit(display, pipe, flags, (void *)(unsigned long)i, fencing);
 		drmHandleEvent(display->drm_fd, &drm_events);
 
 		if (type == TRANSITION_MODESET_DISABLE) {
@@ -239,8 +493,7 @@ run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output
 
 			wm_setup_plane(display, pipe, 0, parms);
 
-			igt_display_commit_atomic(display, flags, (void *)0UL);
-
+			atomic_commit(display, pipe, flags, (void *) 0UL, fencing);
 			drmHandleEvent(display->drm_fd, &drm_events);
 		} else {
 			uint32_t j;
@@ -252,20 +505,23 @@ run_transition_test(igt_display_t *display, enum pipe pipe, igt_output_t *output
 				if (type == TRANSITION_MODESET)
 					igt_output_override_mode(output, &override_mode);
 
-				igt_display_commit_atomic(display, flags, (void *)(unsigned long)j);
+				atomic_commit(display, pipe, flags, (void *)(unsigned long) j, fencing);
 				drmHandleEvent(display->drm_fd, &drm_events);
 
 				wm_setup_plane(display, pipe, i, parms);
 				if (type == TRANSITION_MODESET)
 					igt_output_override_mode(output, NULL);
 
-				igt_display_commit_atomic(display, flags, (void *)(unsigned long)i);
+				atomic_commit(display, pipe, flags, (void *)(unsigned long) i, fencing);
 				drmHandleEvent(display->drm_fd, &drm_events);
 			}
 		}
 	}
 
 cleanup:
+	if (fencing)
+		unprepare_fencing(display, pipe);
+
 	igt_output_set_pipe(output, PIPE_NONE);
 
 	for_each_plane_on_pipe(display, pipe, plane)
@@ -275,6 +531,7 @@ cleanup:
 
 	igt_remove_fb(display->drm_fd, &fb);
 	igt_remove_fb(display->drm_fd, &argb_fb);
+	igt_remove_fb(display->drm_fd, &sprite_fb);
 	if (skip_test)
 		igt_skip("Atomic nonblocking modesets are not supported.\n");
 }
@@ -291,33 +548,30 @@ static void commit_display(igt_display_t *display, unsigned event_mask, bool non
 
 	igt_display_commit_atomic(display, flags, NULL);
 
+	igt_debug("Event mask: %x, waiting for %i events\n", event_mask, num_events);
+
+	igt_set_timeout(30, "Waiting for events timed out\n");
+
 	while (num_events) {
 		char buf[32];
 		struct drm_event *e = (void *)buf;
 		struct drm_event_vblank *vblank = (void *)buf;
-		uint32_t crtc_id, pipe = I915_MAX_PIPES;
 
+		igt_set_timeout(3, "Timed out while reading drm_fd\n");
 		ret = read(display->drm_fd, buf, sizeof(buf));
+		igt_reset_timeout();
 		if (ret < 0 && (errno == EINTR || errno == EAGAIN))
 			continue;
 
 		igt_assert(ret >= 0);
 		igt_assert_eq(e->type, DRM_EVENT_FLIP_COMPLETE);
 
-		crtc_id = vblank->reserved;
-		if (crtc_id) {
-			for_each_pipe(display, pipe)
-				if (display->pipes[pipe].crtc_id == crtc_id)
-					break;
-
-			igt_assert_lt(pipe, display->n_pipes);
-
-			igt_debug("Retrieved vblank seq: %u on %u/%u\n", vblank->sequence, vblank->reserved, pipe);
-		} else
-			igt_debug("Retrieved vblank seq: %u on unk/unk\n", vblank->sequence);
+		igt_debug("Retrieved vblank seq: %u on unk/unk\n", vblank->sequence);
 
 		num_events--;
 	}
+
+	igt_reset_timeout();
 }
 
 static unsigned set_combinations(igt_display_t *display, unsigned mask, struct igt_fb *fb)
@@ -330,7 +584,8 @@ static unsigned set_combinations(igt_display_t *display, unsigned mask, struct i
 		igt_output_set_pipe(output, PIPE_NONE);
 
 	for_each_pipe(display, pipe) {
-		igt_plane_t *plane = &display->pipes[pipe].planes[IGT_PLANE_PRIMARY];
+		igt_plane_t *plane = igt_pipe_get_plane_type(&display->pipes[pipe],
+			DRM_PLANE_TYPE_PRIMARY);
 		drmModeModeInfo *mode = NULL;
 
 		if (!(mask & (1 << pipe))) {
@@ -371,7 +626,7 @@ static void refresh_primaries(igt_display_t *display)
 
 	for_each_pipe(display, pipe)
 		for_each_plane_on_pipe(display, pipe, plane)
-			if (plane->is_primary && plane->fb)
+			if (plane->type == DRM_PLANE_TYPE_PRIMARY && plane->fb)
 				plane->fb_changed = true;
 }
 
@@ -390,12 +645,12 @@ static void collect_crcs_mask(igt_pipe_crc_t **pipe_crcs, unsigned mask, igt_crc
 	}
 }
 
-static void run_modeset_tests(igt_display_t *display, int howmany, bool nonblocking)
+static void run_modeset_tests(igt_display_t *display, int howmany, bool nonblocking, bool fencing)
 {
 	struct igt_fb fbs[2];
 	int i, j;
-	unsigned iter_max = 1 << I915_MAX_PIPES;
-	igt_pipe_crc_t *pipe_crcs[I915_MAX_PIPES];
+	unsigned iter_max = 1 << display->n_pipes;
+	igt_pipe_crc_t *pipe_crcs[I915_MAX_PIPES] = { 0 };
 	igt_output_t *output;
 	unsigned width = 0, height = 0;
 	bool skip_test = false;
@@ -415,7 +670,8 @@ static void run_modeset_tests(igt_display_t *display, int howmany, bool nonblock
 				    DRM_FORMAT_XRGB8888, 0, .5, .5, .5, &fbs[1]);
 
 	for_each_pipe(display, i) {
-		igt_plane_t *plane = &display->pipes[i].planes[IGT_PLANE_PRIMARY];
+		igt_pipe_t *pipe = &display->pipes[i];
+		igt_plane_t *plane = igt_pipe_get_plane_type(pipe, DRM_PLANE_TYPE_PRIMARY);
 		drmModeModeInfo *mode = NULL;
 
 		if (is_i915_device(display->drm_fd))
@@ -436,6 +692,9 @@ static void run_modeset_tests(igt_display_t *display, int howmany, bool nonblock
 			igt_plane_set_size(plane, mode->hdisplay, mode->vdisplay);
 		} else
 			igt_plane_set_fb(plane, NULL);
+
+		if(fencing)
+			igt_pipe_request_out_fence(&display->pipes[i]);
 	}
 
 	/*
@@ -523,7 +782,7 @@ cleanup:
 
 }
 
-static void run_modeset_transition(igt_display_t *display, int requested_outputs, bool nonblocking)
+static void run_modeset_transition(igt_display_t *display, int requested_outputs, bool nonblocking, bool fencing)
 {
 	igt_output_t *outputs[I915_MAX_PIPES] = {};
 	int num_outputs = 0;
@@ -551,7 +810,7 @@ static void run_modeset_transition(igt_display_t *display, int requested_outputs
 		      "Should have at least %i outputs, found %i\n",
 		      requested_outputs, num_outputs);
 
-	run_modeset_tests(display, requested_outputs, nonblocking);
+	run_modeset_tests(display, requested_outputs, nonblocking, fencing);
 }
 
 igt_main
@@ -580,28 +839,50 @@ igt_main
 		igt_require_f(valid_outputs, "no valid crtc/connector combinations found\n");
 	}
 
+	igt_subtest("plane-primary-toggle-with-vblank-wait")
+		for_each_pipe_with_valid_output(&display, pipe, output)
+			run_primary_test(&display, pipe, output);
+
 	igt_subtest("plane-all-transition")
 		for_each_pipe_with_valid_output(&display, pipe, output)
-			run_transition_test(&display, pipe, output, TRANSITION_PLANES, false);
+			run_transition_test(&display, pipe, output, TRANSITION_PLANES, false, false);
+
+	igt_subtest("plane-all-transition-fencing")
+		for_each_pipe_with_valid_output(&display, pipe, output)
+			run_transition_test(&display, pipe, output, TRANSITION_PLANES, false, true);
 
 	igt_subtest("plane-all-transition-nonblocking")
 		for_each_pipe_with_valid_output(&display, pipe, output)
-			run_transition_test(&display, pipe, output, TRANSITION_PLANES, true);
+			run_transition_test(&display, pipe, output, TRANSITION_PLANES, true, false);
+
+	igt_subtest("plane-all-transition-nonblocking-fencing")
+		for_each_pipe_with_valid_output(&display, pipe, output)
+			run_transition_test(&display, pipe, output, TRANSITION_PLANES, true, true);
 
 	igt_subtest("plane-all-modeset-transition")
 		for_each_pipe_with_valid_output(&display, pipe, output)
-			run_transition_test(&display, pipe, output, TRANSITION_MODESET, false);
+			run_transition_test(&display, pipe, output, TRANSITION_MODESET, false, false);
+
+	igt_subtest("plane-all-modeset-transition-fencing")
+		for_each_pipe_with_valid_output(&display, pipe, output)
+			run_transition_test(&display, pipe, output, TRANSITION_MODESET, false, true);
 
 	igt_subtest("plane-toggle-modeset-transition")
 		for_each_pipe_with_valid_output(&display, pipe, output)
-			run_transition_test(&display, pipe, output, TRANSITION_MODESET_DISABLE, false);
+			run_transition_test(&display, pipe, output, TRANSITION_MODESET_DISABLE, false, false);
 
 	for (i = 1; i <= I915_MAX_PIPES; i++) {
 		igt_subtest_f("%ix-modeset-transitions", i)
-			run_modeset_transition(&display, i, false);
+			run_modeset_transition(&display, i, false, false);
 
 		igt_subtest_f("%ix-modeset-transitions-nonblocking", i)
-			run_modeset_transition(&display, i, true);
+			run_modeset_transition(&display, i, true, false);
+
+		igt_subtest_f("%ix-modeset-transitions-fencing", i)
+			run_modeset_transition(&display, i, false, true);
+
+		igt_subtest_f("%ix-modeset-transitions-nonblocking-fencing", i)
+			run_modeset_transition(&display, i, true, true);
 	}
 
 	igt_fixture {
