@@ -1018,9 +1018,10 @@ fb_is_bound(struct test_output *o, int fb)
 	int n;
 
 	for (n = 0; n < o->count; n++) {
-		struct drm_mode_crtc mode;
+		struct drm_mode_crtc mode = {
+			.crtc_id = o->_crtc[n]
+		};
 
-		mode.crtc_id = o->_crtc[n];
 		if (drmIoctl(drm_fd, DRM_IOCTL_MODE_GETCRTC, &mode))
 			return 0;
 
@@ -1186,7 +1187,22 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 	for (n = 0; n < CALIBRATE_TS_STEPS; n++) {
 		struct drm_event_vblank ev;
 		uint64_t now;
+		int poll_ret;
 
+		while (1) {
+			/*
+			 * In case of the interruptible tests, this poll may
+			 * be interrupted with -EINTR, handle this by restarting
+			 * until we poll timeout or success.
+			 */
+			poll_ret = poll(&(struct pollfd){drm_fd, POLLIN}, 1, -1);
+
+			if (poll_ret == 1)
+				break;
+
+			igt_assert_neq(poll_ret, 0);
+			igt_assert_eq(errno, EINTR);
+		}
 		igt_assert(read(drm_fd, &ev, sizeof(ev)) == sizeof(ev));
 		igt_assert_eq(ev.sequence, last_seq + 1);
 
@@ -1206,9 +1222,12 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 	stddev = igt_stats_get_std_deviation(&stats);
 
 	igt_info("Expected frametime: %.0fus; measured %.1fus +- %.3fus accuracy %.2f%%\n",
-		 expected, mean, stddev, 100 * 6 * stddev / mean);
-	igt_assert(6 * stddev / mean < 0.005); /* 99% accuracy within 0.5% */
-
+		 expected, mean, stddev, 100 * 3 * stddev / mean);
+	/* 99.7% samples within 0.5% of the mean */
+	igt_assert(3 * stddev / mean < 0.005);
+	/* 84% samples within 0.5% of the expected value.
+	 * See comments in check_timings() in kms_setmode.c
+	 */
 	if (fabs(mean - expected) > 2*stddev) {
 		igt_warn("vblank interval differs from modeline! expected %.1fus, measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma)\n",
 				expected, mean, stddev,
