@@ -515,7 +515,7 @@ static void poll_dsl_deiir(uint32_t devid, int pipe, int bit,
 	bit = 1 << bit;
 	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
-	if (IS_GEN8(devid)) {
+	if (intel_gen(devid) >= 8) {
 		iir = GEN8_DE_PIPE_IIR(pipe);
 		ier = GEN8_DE_PIPE_IER(pipe);
 		imr = GEN8_DE_PIPE_IMR(pipe);
@@ -961,7 +961,7 @@ static void __attribute__((noreturn)) usage(const char *name)
 
 int main(int argc, char *argv[])
 {
-	int fd, i;
+	int i;
 	int pipe = 0, bit = 0, target_scanline = 0, target_fuzz = 1;
 	bool test_pixelcount = false;
 	uint32_t devid;
@@ -1045,9 +1045,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	fd = drm_open_driver(DRIVER_INTEL);
-	devid = intel_get_drm_devid(fd);
-	close(fd);
+	devid = intel_get_pci_device()->device_id;
 
 	/*
 	 * check if the requires registers are
@@ -1154,7 +1152,7 @@ int main(int argc, char *argv[])
 		default:
 			usage(argv[0]);
 		}
-	} else if (IS_GEN8(devid)) {
+	} else if (intel_gen(devid) >= 8) {
 		if (test_pixelcount)
 			usage(argv[0]);
 
